#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System.Configuration;

namespace Weapsy
{

	public class AppSettings
	{

		#region Private Methods

		private static string GetValue(string Key)
		{
			string Value = ConfigurationManager.AppSettings[Key];
			if (!string.IsNullOrEmpty(Value)) {
				return Value;
			}
			return string.Empty;
		}

		private static string GetString(string Key, string DefaultValue)
		{
			string Setting = GetValue(Key);
			if (!string.IsNullOrEmpty(Setting)) {
				return Setting;
			}
			return DefaultValue;
		}

		private static bool GetBool(string Key, bool DefaultValue)
		{
			string Setting = GetValue(Key);
			if (!string.IsNullOrEmpty(Setting)) {
				switch (Setting.ToLower()) {
					case "false":
					case "0":
					case "n":
						return false;
					case "true":
					case "1":
					case "y":
						return true;
				}
			}
			return DefaultValue;
		}

		private static int GetInt(string Key, int DefaultValue)
		{
			string Setting = GetValue(Key);
			if (!string.IsNullOrEmpty(Setting)) {
                int i;
				if (int.TryParse(Setting, out i)) {
					return i;
				}
			}
			return DefaultValue;
		}

		private static double GetDouble(string Key, double DefaultValue)
		{
			string Setting = GetValue(Key);
			if (!string.IsNullOrEmpty(Setting)) {
                double d;
                if (double.TryParse(Setting, out d)) {
					return d;
				}
			}
			return DefaultValue;
		}

        private static byte GetByte(string Key, byte DefaultValue)
        {
            string Setting = GetValue(Key);
            if (!string.IsNullOrEmpty(Setting))
            {
                byte b;
                if (byte.TryParse(Setting, out b))
                {
                    return b;
                }
            }
            return DefaultValue;
        }

		#endregion

		#region Public Properties

		public static string SiteName {
			get { return GetString("SiteName", "Default"); }
		}

        public static string DefaultLanguage
        {
            get { return GetString("DefaultLanguage", "en-US"); }
        }

        public static string DefaultTheme
        {
            get { return GetString("DefaultTheme", "Default"); }
        }

        public static string DefaultPageTemplate
        {
            get { return GetString("DefaultPageTemplate", "Header-Content-Footer"); }
        }

        public static string DefaultModuleTemplate
        {
            get { return GetString("DefaultModuleTemplate", "Standard"); }
        }

        public static string DefaultMembershipName
        {
            get { return GetString("DefaultMembershipName", "Email"); }
        }

        public static string FolderForFiles
        {
            get { return GetString("FolderForFiles", "~/files/"); }
        }

        public static string RoleSuperAdmin
        {
            get { return GetString("RoleSuperAdmin", "SuperAdmin"); }
        }

        public static string RoleAdmin
        {
            get { return GetString("RoleAdmin", "Admin"); }
        }

        public static string UserSuperAdminName
        {
            get { return GetString("UserSuperAdminName", "superadmin"); }
        }

        public static string UserAdminName
        {
            get { return GetString("UserAdminName", "admin"); }
        }

        public static string UserSuperAdminEmail
        {
            get { return GetString("UserSuperAdminEmail", "superadmin@email.com"); }
        }

        public static string UserAdminEmail
        {
            get { return GetString("UserAdminEmail", "admin@email.com"); }
        }

        //public static double CacheSettingsTimeOut
        //{
        //    get { return GetDouble("CacheSettingsTimeOut", 600); }
        //}

        //public static byte CacheSettingsPriority
        //{
        //    get { return GetByte("CacheSettingsPriority", 3); }
        //}

		#endregion

	}

}