#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Net;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Web;

namespace Weapsy
{

	public class Thumbnail : IHttpHandler
	{

		public void ProcessRequest(HttpContext context)
		{
			string strPath = context.Request.QueryString["p"];
			string strWidth = context.Request.QueryString["w"];
			string strHeight = context.Request.QueryString["h"];

			try {
				if (!string.IsNullOrEmpty(strPath)) {
					if (string.IsNullOrEmpty(strWidth)) {
						strWidth = "100";
					}
					Image image = this.GetImage(strPath, context);
					ImageFormat rawFormat = image.RawFormat;
					Size size = this.ThumbSize(image.Width, image.Height, Convert.ToInt16(strWidth));
					Bitmap bitmap = new Bitmap(size.Width, size.Height);
					Graphics graphics = System.Drawing.Graphics.FromImage(bitmap);
					graphics.SmoothingMode = SmoothingMode.HighQuality;
					graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;
					graphics.DrawImage(image, 0, 0, size.Width, size.Height);
					graphics.Dispose();
					context.Response.ContentType = this.GetContentType(rawFormat);
					MemoryStream stream = new MemoryStream();
					bitmap.Save(stream, ImageFormat.Jpeg);
					stream.WriteTo(context.Response.OutputStream);
					image.Dispose();
					bitmap.Dispose();
				}
			} catch (Exception exception) {
				string message = exception.Message;
			}
		}

		private Image GetImage(string strPath, HttpContext context)
		{
			if ((strPath.IndexOf("http://") > -1) | (strPath.IndexOf("https://") > -1)) {
				try {
					HttpWebRequest request = (HttpWebRequest)WebRequest.Create(strPath);
					request.Credentials = CredentialCache.DefaultCredentials;
					HttpWebResponse response = (HttpWebResponse)request.GetResponse();
					return System.Drawing.Image.FromStream(response.GetResponseStream());
				} catch {
					return Image.FromFile(context.Server.MapPath("~/Content/Images/thumbnail.jpg"));
				}
			}
			return Image.FromFile(context.Server.MapPath(strPath));
		}

		private Size ThumbSize(int currentWidth, int currentHeight, int newWidth)
		{
			double num = 0;
			if ((currentWidth > newWidth)) {
				num = (Convert.ToDouble(newWidth) / Convert.ToDouble(currentWidth));
			} else {
				num = 1;
			}
			return new Size(Convert.ToInt16(Convert.ToDouble((currentWidth * num))), Convert.ToInt16(Convert.ToDouble((currentHeight * num))));
		}

		private string GetContentType(ImageFormat imgF)
		{
			if (imgF.Equals(ImageFormat.Jpeg)) {
				return "image/jpeg";
			}
            if (imgF.Equals(ImageFormat.Png))
            {
				return "image/png";
			}
            if (imgF.Equals(ImageFormat.Bmp))
            {
				return "image/bmp";
			}
            if (imgF.Equals(ImageFormat.Gif))
            {
				return "image/gif";
			}
			return "";
		}

		public bool IsReusable {
			get { return true; }
		}

	}

}
