#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Data.Objects.DataClasses;
using System.Data.Entity;
using System.Data.Entity.Validation;
using System.Linq;
using System.Linq.Expressions;
using Weapsy.Entities;

namespace Weapsy.Data
{
    public class EfRepository<T> : IRepository<T> where T : BaseEntity
    {

        private readonly IDbContext _context;
        private readonly IDbSet<T> _entities;

        public EfRepository(IDbContext context)
        {
            _context = context;
            _entities = context.Set<T>();
        }

        public void Create(T entity)
        {
            try
            {
                if (entity == null)
                {
                    throw new ArgumentNullException("entity");
                }
                _entities.Add(entity);
                _context.SaveChanges();
            }
            catch (DbEntityValidationException dbEx)
            {
                string msg = string.Empty;
                foreach (DbEntityValidationResult dbevr in dbEx.EntityValidationErrors)
                {
                    foreach (DbValidationError dbve in dbevr.ValidationErrors)
                    {
                        msg = msg + string.Format("Property: {0} - Error: {1}", dbve.PropertyName, dbve.ErrorMessage) + Environment.NewLine;
                    }
                }
                Exception fail = new Exception(msg, dbEx);
                throw fail;
            }
        }

        public void Update(T entity)
        {
            try
            {
                if (entity == null)
                {
                    throw new ArgumentNullException("entity");
                }
                _context.SaveChanges();
            }
            catch (DbEntityValidationException dbEx)
            {
                string msg = string.Empty;
                foreach (DbEntityValidationResult dbevr in dbEx.EntityValidationErrors)
                {
                    foreach (DbValidationError dbve in dbevr.ValidationErrors)
                    {
                        msg = msg + string.Format("Property: {0} - Error: {1}", dbve.PropertyName, dbve.ErrorMessage) + Environment.NewLine;
                    }
                }
                Exception fail = new Exception(msg, dbEx);
                throw fail;
            }
        }

        public void Delete(T entity)
        {
            try
            {
                if (entity == null)
                {
                    throw new ArgumentNullException("entity");
                }
                _entities.Remove(entity);
                _context.SaveChanges();
            }
            catch (DbEntityValidationException dbEx)
            {
                string msg = string.Empty;
                foreach (DbEntityValidationResult dbevr in dbEx.EntityValidationErrors)
                {
                    foreach (DbValidationError dbve in dbevr.ValidationErrors)
                    {
                        msg = msg + string.Format("Property: {0} - Error: {1}", dbve.PropertyName, dbve.ErrorMessage) + Environment.NewLine;
                    }
                }
                Exception fail = new Exception(msg, dbEx);
                throw fail;
            }
        }

        public virtual IQueryable<T> Table
        {
			get { return _entities; }
        }

    }

}