﻿#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Data.Entity.ModelConfiguration;
using Weapsy.Entities;

namespace Weapsy.Data.Mapping
{
    public partial class PageMap : EntityTypeConfiguration<Page>
    {
        public PageMap()
        {
            this.ToTable("Page");

            this.HasKey(p => p.Id);

            this.Property(p => p.SiteId).IsRequired();
            this.Property(p => p.ParentId).IsRequired();
            this.Property(p => p.Name).IsRequired().HasMaxLength(100);
            this.Property(p => p.Url).HasMaxLength(200);
            this.Property(p => p.PageTitle).HasMaxLength(100);
            this.Property(p => p.PageIcon).HasMaxLength(256);
            this.Property(p => p.PageImage).HasMaxLength(256);
            this.Property(p => p.MenuText).HasMaxLength(100);
            this.Property(p => p.MenuTitle).HasMaxLength(250);
            this.Property(p => p.MenuIcon).HasMaxLength(256);
            this.Property(p => p.MenuImage).HasMaxLength(256);
            this.Property(p => p.MetaTitle).HasMaxLength(250);
            this.Property(p => p.MetaDescription).HasMaxLength(500);
            this.Property(p => p.MetaKeywords).HasMaxLength(500);
            this.Property(p => p.PageTemplateId).IsRequired();
            this.Property(p => p.ModuleTemplateId).IsRequired();
            this.Property(p => p.Authorized).IsRequired().HasMaxLength(1000);
            this.Property(p => p.MainMenu).IsRequired();
            this.Property(p => p.SortOrder).IsRequired();
            this.Property(p => p.Active).IsRequired();
            this.Property(p => p.Deleted).IsRequired();
            this.Property(p => p.StartDate);
            this.Property(p => p.EndDate);
            this.Property(p => p.DisplayPageTitle).IsRequired();
            this.Property(p => p.LanguageUrlEnabled).IsRequired();
            this.Property(p => p.SiteMap).IsRequired();
            this.Property(p => p.Link).HasMaxLength(256);
            this.Property(p => p.Disabled).IsRequired();
            this.Property(p => p.CreatedByUserId).IsRequired();
            this.Property(p => p.CreatedByIpAddress).IsRequired().HasMaxLength(15);
            this.Property(p => p.CreatedOnDate).IsRequired();
            this.Property(p => p.ModifiedByUserId).IsRequired();
            this.Property(p => p.ModifiedByIpAddress).IsRequired().HasMaxLength(15);
            this.Property(p => p.ModifiedOnDate).IsRequired();

            //this.Property(p => p.UseModules).IsRequired();
            //this.Property(p => p.Body).IsMaxLength();

            this.HasRequired(p => p.Site)
                .WithMany(s => s.Pages)
                .HasForeignKey(p => p.SiteId)
                .WillCascadeOnDelete(false);

            //this.HasOptional(p => p.PageTemplate)
            //    .WithMany(pt => pt.Pages)
            //    .HasForeignKey(p => p.PageTemplateId);

            //this.HasOptional(p => p.ModuleTemplate)
            //    .WithMany(mt => mt.Pages)
            //    .HasForeignKey(p => p.ModuleTemplateId);

            this.Ignore(p => p.PageModuleCount);
            this.Ignore(p => p.LocalizedUrl);
            this.Ignore(p => p.LocalizedPageTitle);
            this.Ignore(p => p.LocalizedPageImage);
            this.Ignore(p => p.LocalizedMenuText);
            this.Ignore(p => p.LocalizedMenuTitle);
            this.Ignore(p => p.LocalizedMenuImage);
            this.Ignore(p => p.LocalizedMetaTitle);
            this.Ignore(p => p.LocalizedMetaDescription);
            this.Ignore(p => p.LocalizedMetaKeywords);
            this.Ignore(p => p.NameForTree);
        }
    }
}