﻿#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Data.Entity.ModelConfiguration;
using System.Data.Entity.ModelConfiguration.Conventions;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Web.Management;
using Weapsy.Data.Initializers;
using Weapsy.Data.Mapping;
using Weapsy.Entities;
using Weapsy.Apps.Forum.Data.Mapping;

namespace Weapsy.Data
{
    public partial class WeapsyContext : DbContext, IDbContext
    {
        public WeapsyContext()
            : base("name=WeapsyConnection")
        {
            if (AppDomain.CurrentDomain.IsFullyTrusted)
            {
                //Database.SetInitializer(new MigrationInitializer());
                Database.SetInitializer(new DropCreateInitializer());
            }
            else
            {
                Database.SetInitializer(new ValidateDatabaseInitializer<WeapsyContext>());
            }
        }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            modelBuilder.Conventions.Remove<PluralizingTableNameConvention>();

            modelBuilder.Configurations.Add(new AppMap());
            modelBuilder.Configurations.Add(new EmailAccountMap());
            modelBuilder.Configurations.Add(new LanguageMap());
            modelBuilder.Configurations.Add(new LocalizedMailTemplateMap());
            modelBuilder.Configurations.Add(new LocalizedModuleMap());
            modelBuilder.Configurations.Add(new LocalizedPageMap());
            modelBuilder.Configurations.Add(new LocalizedPageModuleMap());
            modelBuilder.Configurations.Add(new LocalizedPluginMap());
            modelBuilder.Configurations.Add(new LocalizedSiteMap());
            modelBuilder.Configurations.Add(new LocalizedTextVersionMap());
            modelBuilder.Configurations.Add(new LocalizedWidgetMap());
            modelBuilder.Configurations.Add(new MailTemplateMap());
            modelBuilder.Configurations.Add(new MailTemplateTypeMap());
            modelBuilder.Configurations.Add(new ModuleDefinitionMap());
            modelBuilder.Configurations.Add(new ModuleMap());
            modelBuilder.Configurations.Add(new ModuleTemplateMap());
            modelBuilder.Configurations.Add(new PageMap());
            modelBuilder.Configurations.Add(new PageModuleMap());
            modelBuilder.Configurations.Add(new PageTemplateMap());
            modelBuilder.Configurations.Add(new PluginDefinitionMap());
            modelBuilder.Configurations.Add(new PluginMap());
            modelBuilder.Configurations.Add(new SettingMap());
            modelBuilder.Configurations.Add(new SiteMap());
            modelBuilder.Configurations.Add(new TextMap());
            modelBuilder.Configurations.Add(new TextVersionMap());
            modelBuilder.Configurations.Add(new ThemeMap());
            modelBuilder.Configurations.Add(new UserMap());
            modelBuilder.Configurations.Add(new WidgetDefinitionMap());
            modelBuilder.Configurations.Add(new WidgetMap());

            // FORUM APP
            modelBuilder.Configurations.Add(new ForumCategoryMap());
            modelBuilder.Configurations.Add(new ForumConfigurationMap());
            modelBuilder.Configurations.Add(new ForumPermissionSetMap());
            modelBuilder.Configurations.Add(new ForumPostMap());
            modelBuilder.Configurations.Add(new ForumSectionMap());
            modelBuilder.Configurations.Add(new ForumSettingSetMap());
            modelBuilder.Configurations.Add(new ForumUserMap());

            base.OnModelCreating(modelBuilder);
        }

        public new IDbSet<TEntity> Set<TEntity>() where TEntity : BaseEntity
        {
            return base.Set<TEntity>();
        }

    }
}