#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
    public class AppService : IAppService
    {

        #region Private Members

        private readonly IRepository<App> _appRepository;
        private readonly ICacheManager _cacheManager;
        private readonly ISettingService _settingService;

        private const string Key = Consts.GlobalKey + "|Apps";

        #endregion

        #region Constructors

        public AppService(IRepository<App> appRepository,
            ICacheManager cacheManager,
            ISettingService settingService)
        {
            _appRepository = appRepository;
            _cacheManager = cacheManager;
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public virtual App Create(App entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("App");
            }
            _appRepository.Create(entity);
            _cacheManager.Remove(Key);
            return entity;
        }

        public virtual void Update(App entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("App");
            }
            _appRepository.Update(entity);
            _cacheManager.Remove(Key);
        }

        public virtual void Delete(App entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("App");
            }

            _appRepository.Delete(entity);

            _cacheManager.Remove(Key);
        }

        public virtual App GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual App GetById(int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(Id);
            }
        }

        public virtual App GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual App GetByName(string Name, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Name == Name);
            }
            else
            {
                return _GetByName(Name);
            }
        }

        public virtual List<App> GetList()
        {
            return GetList(true);
        }

        public virtual List<App> GetList(bool UseCache)
        {
            List<App> list;
            if (UseCache)
            {
                object obj = _cacheManager.Get(Key);
                if (obj != null)
                {
                    list = (List<App>)obj;
                }
                else
                {
                    list = _GetList();
                    _cacheManager.Insert(Key, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList();
            }
            return list;
        }

        public virtual Dictionary<string, string> GetDictionary()
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add(Consts.NullId.ToString(), "");
            foreach (var item in GetList())
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }
            return dic;
        }

        #endregion

        #region Private Methods

        private App _GetById(int id)
        {
            var query = from item in _appRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private App _GetByName(string name)
        {
            var query = from item in _appRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private List<App> _GetList()
        {
            var query = from item in _appRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

    }
}