#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class EmailAccountService : IEmailAccountService
    {

        #region Private Members

        private readonly IRepository<EmailAccount> _emailAccountRepository;
        private readonly ICacheManager _cacheManager;
        private readonly ISettingService _settingService;

        private const string CacheKey = Consts.GlobalKey + "|EmailAccounts|SiteId:{0}";

        #endregion

        #region Constructors

        public EmailAccountService(IRepository<EmailAccount> emailAccountRepository,
            ICacheManager cacheManager,
            ISettingService settingService)
        {
            _emailAccountRepository = emailAccountRepository;
            _cacheManager = cacheManager;
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public virtual void Create(EmailAccount entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("EmailAccount");
            }

            _emailAccountRepository.Create(entity);

            _cacheManager.Remove(string.Format(CacheKey, entity.SiteId));
        }

        public virtual void Update(EmailAccount entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("EmailAccount");
            }

            _emailAccountRepository.Update(entity);

            _cacheManager.Remove(string.Format(CacheKey, entity.SiteId));
        }

        public virtual void Delete(EmailAccount entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("EmailAccount");
            }

            _emailAccountRepository.Delete(entity);

            _cacheManager.Remove(string.Format(CacheKey, entity.SiteId));
        }

        public virtual EmailAccount GetById(int siteId, int id)
        {
            return GetById(siteId, id, true);
        }

        public virtual EmailAccount GetById(int siteId, int id, bool useCache)
        {
            if (useCache)
            {
                return GetList(siteId, true).Find(o => o.Id == id);
            }
            else
            {
                return _GetById(siteId, id);
            }
        }

        public virtual List<EmailAccount> GetList(int SiteId)
        {
            return GetList(SiteId, true);
        }

        public virtual List<EmailAccount> GetList(int SiteId, bool UseCache)
        {
            List<EmailAccount> list = new List<EmailAccount>();
            if (UseCache)
            {
                string key = string.Format(CacheKey, SiteId);
                object obj = _cacheManager.Get(key);
                if (obj != null)
                {
                    list = (List<EmailAccount>)obj;
                }
                else
                {
                    list = _GetList(SiteId);
                    _cacheManager.Insert(key, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList(SiteId);
            }
            return list;
        }

        #endregion

        #region Private Methods

        private EmailAccount _GetById(int siteId, int id)
        {
            var query = from item in _emailAccountRepository.Table
                        where item.SiteId == siteId
                        & item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private List<EmailAccount> _GetList(int siteId)
        {
            var query = from item in _emailAccountRepository.Table
                        where item.SiteId == siteId
                        orderby item.DisplayName
                        select item;
            return query.ToList();
        }

        #endregion

	}

}