#region Copyright

//  ProjectXCS - http://projectxcs.codeplex.com
//  Copyright (c) 2011 Luca Cammarata Briguglia
//  Licensed under the GNU General Public License, version 2 (GPL-2.0)
//  A copy of this license may be found at http://projectxcs.codeplex.com/license

#endregion

using System.Configuration.Provider;
using System.Web.Configuration;
using PagedList;

namespace ProjectXCS.BusinessLogic
{

	public class Friendships
	{

		public static void Create(int FromUserId, int ToUserId)
		{
			using (Models.Entities db = new Models.Entities()) {
				Models.Friendship model = new Models.Friendship();

				model.CreatedByUserId = FromUserId;
				model.CreatedByIPAddress = HttpContext.Current.Request.UserHostAddress;
				model.CreatedOnDate = DateTime.UtcNow;

				model.ReceivedByUserId = ToUserId;

				db.Friendships.AddObject(model);

				db.SaveChanges();
			}

			CacheService.ClearUserPendingFriends(FromUserId);
			CacheService.ClearUserPendingFriends(ToUserId);
		}

		public static void Delete(int FriendshipId)
		{
			using (Models.Entities db = new Models.Entities()) {
				Models.Friendship obj = db.Friendships.First((System.Object o) => o.FriendshipId == FriendshipId);
				db.Friendships.DeleteObject(obj);
				db.SaveChanges();
				CacheService.ClearUserFriends(obj.CreatedByUserId);
				CacheService.ClearUserFriends(obj.ReceivedByUserId);
			}
		}

		public static void Delete(int UserId1, int UserId2)
		{
			using (Models.Entities db = new Models.Entities()) {
				Models.Friendship obj = db.Friendships.First((System.Object o) => (o.CreatedByUserId == UserId1 & o.ReceivedByUserId == UserId2) | (o.CreatedByUserId == UserId2 & o.ReceivedByUserId == UserId1));
				db.Friendships.DeleteObject(obj);
				db.SaveChanges();
				CacheService.ClearUserPendingFriends(UserId1);
				CacheService.ClearUserPendingFriends(UserId2);
			}
		}

		public static Models.Friendship GetFriendship(int FriendshipId)
		{
			return new Models.Entities().Friendships.FirstOrDefault((System.Object o) => o.FriendshipId == FriendshipId);
		}

		public static Models.Friendship GetFriendship(int UserId1, int UserId2)
		{
			return new Models.Entities().Friendships.FirstOrDefault((System.Object o) => (o.CreatedByUserId == UserId1 & o.ReceivedByUserId == UserId2) | (o.CreatedByUserId == UserId2 & o.ReceivedByUserId == UserId1));
		}

		//Public Shared Function GetFriends(ByVal UserId As Integer, ByVal Approved As Boolean) As List(Of Models.Friendship)
		//    Using db As New Models.Entities
		//        Dim items = From item In db.Friendships _
		//                    .Include("User") _
		//                    .Include("User1") _
		//                    .Where(Function(o) (o.CreatedByUserId = UserId Or o.ReceivedByUserId = UserId) And o.Accepted = True) _
		//                    .OrderBy(Function(o) o.FriendshipId) _
		//                    Select item

		//        Dim Result As PagedList.PagedList(Of Models.User) = items.ToPagedList(PageIndex - 1, PageSize)

		//        TotalRecords = Result.TotalItemCount
		//        TotalPages = Result.PageCount

		//        Return Result
		//    End Using
		//End Function

		public static List<Models.Friendship> GetPendingFriends(int UserId)
		{
			List<Models.Friendship> Result = new List<Models.Friendship>();
			foreach (Models.Friendship item in new Models.Entities().Friendships.Where((System.Object o) => (o.CreatedByUserId == UserId | o.ReceivedByUserId == UserId) & o.Accepted == false).ToList()) {
				item.UserId = UserId;
				Result.Add(item);
			}
			return Result;
		}

		public static List<Models.Friendship> GetFriends(int UserId)
		{
			List<Models.Friendship> Result = new List<Models.Friendship>();
			foreach (Models.Friendship item in new Models.Entities().Friendships.Where((System.Object o) => (o.CreatedByUserId == UserId | o.ReceivedByUserId == UserId) & o.Accepted == true).ToList()) {
				item.UserId = UserId;
				Result.Add(item);
			}
			return Result;
		}

		public static PagedList.PagedList<Models.Friendship> GetFriends(int PageIndex, int PageSize, ref int TotalRecords, ref int TotalPages, int UserId)
		{
			using (Models.Entities db = new Models.Entities()) {
				object items = from item in db.Friendships.Include("User").Include("User1").Where((System.Object o) => (o.CreatedByUserId == UserId | o.ReceivedByUserId == UserId) & o.Accepted == true).OrderBy((System.Object o) => o.FriendshipId)item;

				PagedList.PagedList<Models.Friendship> Result = items.ToPagedList(PageIndex - 1, PageSize);

				TotalRecords = Result.TotalItemCount;
				TotalPages = Result.PageCount;

				return Result;
			}
		}

	}

}

//public static bool IsFriend(int UserId)
//{
//    return AreFriends(ProjectContext.Current.User.Id, UserId);
//}

//public static bool AreFriends(int UserId1, int UserId2)
//{
//    foreach (Models.Friendship item in CacheService.GetUserFriends(UserId1)) {
//        if (item.CreatedByUserId == UserId2 | item.ReceivedByUserId == UserId2) {
//            return true;
//        }
//    }
//    return false;
//}

//public static bool IsPendingFriend(int UserId)
//{
//    return ArePendingFriends(ProjectContext.Current.User.Id, UserId);
//}

//public static bool ArePendingFriends(int UserId1, int UserId2)
//{
//    foreach (Models.Friendship item in CacheService.GetUserPendingFriends(UserId1)) {
//        if (item.CreatedByUserId == UserId2 | item.ReceivedByUserId == UserId2) {
//            return true;
//        }
//    }
//    return false;
//}

