#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class MailTemplateTypeService : IMailTemplateTypeService
    {

        #region Private Members

        private readonly IRepository<MailTemplateType> _mailTemplateTypeRepository;
        private readonly ICacheManager _cacheManager;
        private readonly ISettingService _settingService;

        private const string CacheKey = Consts.GlobalKey + "|MailTemplateTypes";

        #endregion

        #region Constructors

        public MailTemplateTypeService(IRepository<MailTemplateType> mailTemplateTypeRepository,
            ICacheManager cacheManager,
            ISettingService settingService)
        {
            _mailTemplateTypeRepository = mailTemplateTypeRepository;
            _cacheManager = cacheManager;
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public virtual void Create(MailTemplateType entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("MailTemplateType");
            }

            _mailTemplateTypeRepository.Create(entity);

            _cacheManager.Remove(CacheKey);
        }

        public virtual void Update(MailTemplateType entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("MailTemplateType");
            }

            _mailTemplateTypeRepository.Update(entity);

            _cacheManager.Remove(CacheKey);
        }

        public virtual void Delete(MailTemplateType entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("MailTemplateType");
            }

            _mailTemplateTypeRepository.Delete(entity);

            _cacheManager.Remove(CacheKey);
        }

        public virtual MailTemplateType GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual MailTemplateType GetById(int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(Id);
            }
        }

        public virtual MailTemplateType GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual MailTemplateType GetByName(string Name, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(m => m.Name == Name);
            }
            else
            {
                return _GetByName(Name);
            }
        }

        public virtual List<MailTemplateType> GetList()
        {
            return GetList(true);
        }

        public virtual List<MailTemplateType> GetList(bool UseCache)
        {
            List<MailTemplateType> list;
            if (UseCache)
            {
                object obj = _cacheManager.Get(CacheKey);
                if (obj != null)
                {
                    list = (List<MailTemplateType>)obj;
                }
                else
                {
                    list = _GetList();
                    _cacheManager.Insert(CacheKey, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList();
            }
            return list;
        }

        public virtual List<MailTemplateType> GetListByAppId(int AppId)
        {
            return GetListByAppId(AppId, true);
        }

        public virtual List<MailTemplateType> GetListByAppId(int AppId, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).FindAll(m => m.AppId == AppId);
            }
            else
            {
                return _GetListByAppId(AppId);
            }
        }

        #endregion

        #region Private Methods

        private MailTemplateType _GetById(int id)
        {
            var query = from item in _mailTemplateTypeRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private MailTemplateType _GetByName(string name)
        {
            var query = from item in _mailTemplateTypeRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private List<MailTemplateType> _GetList()
        {
            var query = from item in _mailTemplateTypeRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        private List<MailTemplateType> _GetListByAppId(int id)
        {
            var query = from item in _mailTemplateTypeRepository.Table
                        where item.AppId == id
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

	}

}