#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class ModuleDefinitionService : IModuleDefinitionService
    {

        #region Private Members

        private readonly IRepository<ModuleDefinition> _moduleDefinitionRepository;
        private readonly IAppService _appService;
        private readonly ICacheManager _cacheManager;
        private readonly ISettingService _settingService;

        private const string CacheKey = Consts.GlobalKey + "|ModuleDefinitions";

        #endregion

        #region Constructors

        public ModuleDefinitionService(IRepository<ModuleDefinition> moduleDefinitionRepository,
            IAppService appService,
            ICacheManager cacheManager,
            ISettingService settingService)
        {
            _moduleDefinitionRepository = moduleDefinitionRepository;
            _appService = appService;
            _cacheManager = cacheManager;
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public virtual void Create(ModuleDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("ModuleDefinition");
            }
            _moduleDefinitionRepository.Create(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Update(ModuleDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("ModuleDefinition");
            }
            _moduleDefinitionRepository.Update(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Delete(ModuleDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("ModuleDefinition");
            }
            _moduleDefinitionRepository.Delete(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual ModuleDefinition GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual ModuleDefinition GetById(int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(Id);
            }
        }

        public virtual ModuleDefinition GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual ModuleDefinition GetByName(string Name, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(m => m.Name == Name);
            }
            else
            {
                return _GetByName(Name);
            }
        }

        public virtual List<ModuleDefinition> GetList()
        {
            return GetList(true);
        }

        public virtual List<ModuleDefinition> GetList(bool UseCache)
        {
            List<ModuleDefinition> list;
            if (UseCache)
            {
                object obj = _cacheManager.Get(CacheKey);
                if (obj != null)
                {
                    list = (List<ModuleDefinition>)obj;
                }
                else
                {
                    list = _GetList();
                    _cacheManager.Insert(CacheKey, list, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                list = _GetList();
            }
            return list;
        }

        public virtual List<ModuleDefinition> GetListByAppId(int AppId)
        {
            return GetListByAppId(AppId, true);
        }

        public virtual List<ModuleDefinition> GetListByAppId(int AppId, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).FindAll(m => m.AppId == AppId);
            }
            else
            {
                return _GetListByAppId(AppId);
            }
        }

        public virtual Dictionary<string, string> GetDictionary()
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            foreach (ModuleDefinition item in GetList())
            {
                dic.Add(item.Id.ToString(), item.Title.ToString());
            }
            return dic;
        }

        #endregion

        #region Private Methods

        private ModuleDefinition _GetById(int id)
        {
            var query = from item in _moduleDefinitionRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private ModuleDefinition _GetByName(string name)
        {
            var query = from item in _moduleDefinitionRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private List<ModuleDefinition> _GetList()
        {
            var query = from item in _moduleDefinitionRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        private List<ModuleDefinition> _GetListByAppId(int id)
        {
            var query = from item in _moduleDefinitionRepository.Table
                        where item.AppId == id
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

	}

}