#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{
    public class ModuleTemplateService : IModuleTemplateService
    {

        #region Private Members

        private readonly IRepository<ModuleTemplate> _moduleTemplateRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|ModuleTemplates";

        #endregion

        #region Constructors

        public ModuleTemplateService(IRepository<ModuleTemplate> moduleTemplateRepository,
            ICacheManager cacheManager)
        {
            _moduleTemplateRepository = moduleTemplateRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(ModuleTemplate entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("ModuleTemplate");
            }
            _moduleTemplateRepository.Create(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Update(ModuleTemplate entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("ModuleTemplate");
            }
            _moduleTemplateRepository.Update(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Delete(ModuleTemplate entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("ModuleTemplate");
            }
            _moduleTemplateRepository.Delete(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual ModuleTemplate GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual ModuleTemplate GetById(int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(Id);
            }
        }

        public virtual ModuleTemplate GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual ModuleTemplate GetByName(string Name, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Name == Name);
            }
            else
            {
                return _GetByName(Name);
            }
        }

        public virtual List<ModuleTemplate> GetList()
        {
            return GetList(true);
        }

        public virtual List<ModuleTemplate> GetList(bool UseCache)
        {
            List<ModuleTemplate> list;
            if (UseCache)
            {
                object obj = _cacheManager.Get(CacheKey);
                if (obj != null)
                {
                    list = (List<ModuleTemplate>)obj;
                }
                else
                {
                    list = _GetList();
                    _cacheManager.Insert(CacheKey, list, 60, CacheItemPriority.Normal);
                }
            }
            else
            {
                list = _GetList();
            }
            return list;
        }

        public virtual Dictionary<string, string> GetDictionary(bool includeInherit)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            if (includeInherit)
            {
                dic.Add(Consts.NullId.ToString(), "<Inherit>");
            }
            foreach (ModuleTemplate item in GetList(true))
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }
            return dic;
        }

        #endregion

        #region Private Methods

        private ModuleTemplate _GetById(int id)
        {
            var query = from item in _moduleTemplateRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private ModuleTemplate _GetByName(string name)
        {
            var query = from item in _moduleTemplateRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private List<ModuleTemplate> _GetList()
        {
            var query = from item in _moduleTemplateRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

    }

}