#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class PluginDefinitionService : IPluginDefinitionService
    {

        #region Private Members

        private readonly IRepository<PluginDefinition> _pluginDefinitionRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|PluginDefinitions";

        #endregion

        #region Constructors

        public PluginDefinitionService(IRepository<PluginDefinition> pluginDefinitionRepository,
            ICacheManager cacheManager)
        {
            _pluginDefinitionRepository = pluginDefinitionRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(PluginDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PluginDefinition");
            }
            _pluginDefinitionRepository.Create(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Update(PluginDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PluginDefinition");
            }
            _pluginDefinitionRepository.Update(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Delete(PluginDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PluginDefinition");
            }
            _pluginDefinitionRepository.Delete(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual PluginDefinition GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual PluginDefinition GetById(int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(Id);
            }
        }

        public virtual PluginDefinition GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual PluginDefinition GetByName(string Name, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(m => m.Name == Name);
            }
            else
            {
                return _GetByName(Name);
            }
        }

        public virtual List<PluginDefinition> GetList()
        {
            return GetList(true);
        }

        public virtual List<PluginDefinition> GetList(bool UseCache)
        {
            List<PluginDefinition> list;
            if (UseCache)
            {
                object obj = _cacheManager.Get(CacheKey);
                if (obj != null)
                {
                    list = (List<PluginDefinition>)obj;
                }
                else
                {
                    list = _GetList();
                    _cacheManager.Insert(CacheKey, list, 60, CacheItemPriority.Normal);
                }
            }
            else
            {
                list = _GetList();
            }
            return list;
        }

        public virtual List<PluginDefinition> GetListByAppId(int AppId)
        {
            return GetListByAppId(AppId, true);
        }

        public virtual List<PluginDefinition> GetListByAppId(int AppId, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).FindAll(m => m.AppId == AppId);
            }
            else
            {
                return _GetListByAppId(AppId);
            }
        }

        #endregion

        #region Private Methods

        private PluginDefinition _GetById(int id)
        {
            var query = from item in _pluginDefinitionRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private PluginDefinition _GetByName(string name)
        {
            var query = from item in _pluginDefinitionRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private List<PluginDefinition> _GetList()
        {
            var query = from item in _pluginDefinitionRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        private List<PluginDefinition> _GetListByAppId(int id)
        {
            var query = from item in _pluginDefinitionRepository.Table
                        where item.AppId == id
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

	}

}