#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using System.Web.Security;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

namespace Weapsy.Services
{

	public class UserService : IUserService
    {

        #region Private Members

        private readonly IRepository<User> _userRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|User|MembershipName:{0}";

        #endregion

        #region Constructors

        public UserService(IRepository<User> userRepository,
            ICacheManager cacheManager)
        {
            _userRepository = userRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(User entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("User");
            }
            _userRepository.Create(entity);
        }

        public virtual void Update(User entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("User");
            }
            _userRepository.Update(entity);
            _cacheManager.Remove(string.Format(CacheKey, entity.MembershipName));
            SessionService.RemoveObject(Consts.SessionKeyUser);
        }

        public virtual void Delete(User entity)
        {
            Delete(entity, true);
        }

        public virtual void Delete(User entity, bool deleteMembership)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("User");
            }
            _userRepository.Delete(entity);
            _cacheManager.Remove(string.Format(CacheKey, entity.MembershipName));
            if (deleteMembership)
            {
                MembershipUser mUser = Membership.GetUser(entity.MembershipName);
                if (mUser != null)
                {
                    Membership.DeleteUser(mUser.UserName, true);
                }
            }
        }

        public virtual User GetByMembershipName(string MembershipName)
        {
            return GetByMembershipName(MembershipName, true);
        }

        public virtual User GetByMembershipName(string MembershipName, bool UseCache)
        {
            User entity;
            if (UseCache)
            {
                string key = string.Format(CacheKey, MembershipName);
                object obj = _cacheManager.Get(key);
                if (obj != null)
                {
                    entity = (User)obj;
                }
                else
                {
                    entity = _GetByMembershipName(MembershipName);
                    _cacheManager.Insert(key, entity, 60, CacheItemPriority.Normal);
                }
            }
            else
            {
                entity = _GetByMembershipName(MembershipName);
            }
            return entity;
        }

        public virtual User GetById(int Id)
        {
            var query = from item in _userRepository.Table
                        where item.Id == Id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual User GetByMembershipEmail(string MembershipEmail)
        {
            var query = from item in _userRepository.Table
                        where item.MembershipEmail == MembershipEmail
                        select item;
            return query.FirstOrDefault();
        }

        public virtual List<User> GetList()
        {
            var query = from item in _userRepository.Table
                        orderby item.DisplayName
                        select item;
            return query.ToList();
        }

        #endregion

        #region Private Methods

        private User _GetByMembershipName(string MembershipName)
        {
            var query = from item in _userRepository.Table
                        where item.MembershipName == MembershipName
                        select item;
            return query.FirstOrDefault();
        }

        #endregion

        

















		public virtual bool EmailExists(string MembershipEmail)
		{
            User u = GetByMembershipEmail(MembershipEmail);
			if (u == null) {
				return false;
			} else {
				return true;
			}
		}

        //public virtual List<Models.User> GetAllUsers()
        //{
        //    return new Models.Entities().Users.OrderBy(o => o.MembershipName).ToList();            
        //}

        //public virtual PagedList.PagedList<Models.User> GetUsers(int PageIndex, int PageSize, ref int TotalRecords, ref int TotalPages, string UserName, string Email, string Role, string Approved, string Locked)
        //{
        //    using (Models.Entities db = new Models.Entities()) {
        //        object items = from item in db.Users.Include("aspnet_Membership").Include("aspnet_Users.aspnet_Roles").OrderBy((System.Object o) => o.UserName)item;

        //        if (!string.IsNullOrEmpty(UserName)) {
        //            items = items.Where((System.Object o) => o.UserName.Contains(UserName));
        //        }

        //        if (!string.IsNullOrEmpty(Email)) {
        //            items = items.Where((System.Object o) => o.aspnet_Membership.Email.Contains(Email));
        //        }

        //        if (!string.IsNullOrEmpty(Role)) {
        //            items = items.Where((System.Object o) => o.aspnet_Users.aspnet_Roles.Any((System.Object x) => x.RoleName == Role));
        //        }

        //        if (!string.IsNullOrEmpty(Approved)) {
        //            items = items.Where((System.Object o) => o.aspnet_Membership.IsApproved == Convert.ToBoolean(Approved));
        //        }

        //        if (!string.IsNullOrEmpty(Locked)) {
        //            items = items.Where((System.Object o) => o.aspnet_Membership.IsLockedOut == Convert.ToBoolean(Locked));
        //        }

        //        PagedList.PagedList<Models.User> Result = items.ToPagedList(PageIndex - 1, PageSize);

        //        TotalRecords = Result.TotalItemCount;
        //        TotalPages = Result.PageCount;

        //        return Result;
        //    }
        //}

        //public virtual void UpdateImage(int UserId, string Image)
        //{
        //    using (Models.Entities db = new Models.Entities()) {
        //        Models.User User = db.Users.FirstOrDefault(o => o.Id == UserId);
        //        if (User != null) {
        //            User.ProfileImage = Image;
        //            db.SaveChanges();               
        //        }
        //    }
        //}

        //public virtual bool Unlock(string MembershipName)
        //{
        //    MembershipUser User = Membership.GetUser(MembershipName);
        //    if (User != null) {
        //        return User.UnlockUser();
        //    }
        //    return false;
        //}

        //public virtual void UpdateUser(ViewModels.Admin.UserModel model)
        //{
        //    using (Models.Entities db = new Models.Entities()) {
        //        Models.User User = db.Users.FirstOrDefault(u => u.Id == model.User.Id);
        //        //if (User != null) {
        //            User.DisplayName = model.User.DisplayName;
        //            //User.ProfileName = model.ProfileName;
        //            //User.FirstName = model.FirstName;
        //            //User.MiddleName = model.MiddleName;
        //            //User.LastName = model.LastName;
        //            //User.City = model.City;
        //            //User.BirthdayDay = model.BirthdayDay;
        //            //User.BirthdayMonth = model.BirthdayMonth;
        //            //User.BirthdayYear = model.BirthdayYear;
        //            //User.Gender = model.Gender;
        //            //User.ProfileViews = model.ProfileViews;
        //            //User.ProfileImage = model.ProfileImage;
        //            //User.ForumTopics = model.ForumTopics;
        //            //User.ForumReplies = model.ForumReplies;
        //            //User.ForumBannedUntil = model.ForumBannedUntil;
        //            //User.MessengerAuthorized = model.MessengerAuthorized;
        //            //User.NameAuthorized = model.NameAuthorized;
        //            //User.CityAuthorized = model.CityAuthorized;
        //            //User.BirthdayAuthorized = model.BirthdayAuthorized;
        //            //User.GenderAuthorized = model.GenderAuthorized;
        //            //User.MessengerNotificationEmail = model.MessengerNotificationEmail;
        //            //User.FriendshipNotificationEmail = model.FriendshipNotificationEmail;

        //            User.ModifiedByUserId = WeapsyContext.Current.User.Id;
        //            User.ModifiedByIpAddress = HttpContext.Current.Request.UserHostAddress;
        //            User.ModifiedOnDate = Utils.GetDateForDB();

        //            db.SaveChanges();                
        //        //}
        //    }

        //    MembershipUser mUser = Membership.GetUser(model.MembershipUser.UserName);

        //    mUser.Email = model.MembershipUser.Email;
        //    mUser.Comment = model.MembershipUser.Comment;
        //    mUser.IsApproved = model.MembershipUser.IsApproved;

        //    Membership.UpdateUser(mUser);

        //}

	}
}