#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

#region Usings

using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Configuration;
using System.Web.Caching;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Entities;

#endregion

namespace Weapsy.Services
{

	public class WidgetDefinitionService : IWidgetDefinitionService
    {

        #region Private Members

        private readonly IRepository<WidgetDefinition> _widgetDefinitionRepository;
        private readonly ICacheManager _cacheManager;

        private const string CacheKey = Consts.GlobalKey + "|WidgetDefinitions";

        #endregion

        #region Constructors

        public WidgetDefinitionService(IRepository<WidgetDefinition> widgetDefinitionRepository,
            ICacheManager cacheManager)
        {
            _widgetDefinitionRepository = widgetDefinitionRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(WidgetDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("WidgetDefinition");
            }
            _widgetDefinitionRepository.Create(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Update(WidgetDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("WidgetDefinition");
            }
            _widgetDefinitionRepository.Update(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual void Delete(WidgetDefinition entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("WidgetDefinition");
            }
            _widgetDefinitionRepository.Delete(entity);
            _cacheManager.Remove(CacheKey);
        }

        public virtual WidgetDefinition GetById(int Id)
        {
            return GetById(Id, true);
        }

        public virtual WidgetDefinition GetById(int Id, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(o => o.Id == Id);
            }
            else
            {
                return _GetById(Id);
            }
        }

        public virtual WidgetDefinition GetByName(string Name)
        {
            return GetByName(Name, true);
        }

        public virtual WidgetDefinition GetByName(string Name, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).Find(m => m.Name == Name);
            }
            else
            {
                return _GetByName(Name);
            }
        }

        public virtual List<WidgetDefinition> GetList()
        {
            return GetList(true);
        }

        public virtual List<WidgetDefinition> GetList(bool UseCache)
        {
            List<WidgetDefinition> list;
            if (UseCache)
            {
                object obj = _cacheManager.Get(CacheKey);
                if (obj != null)
                {
                    list = (List<WidgetDefinition>)obj;
                }
                else
                {
                    list = _GetList();
                    _cacheManager.Insert(CacheKey, list, 60, CacheItemPriority.Normal);
                }
            }
            else
            {
                list = _GetList();
            }
            return list;
        }

        public virtual List<WidgetDefinition> GetListByAppId(int AppId)
        {
            return GetListByAppId(AppId, true);
        }

        public virtual List<WidgetDefinition> GetListByAppId(int AppId, bool UseCache)
        {
            if (UseCache)
            {
                return GetList(true).FindAll(m => m.AppId == AppId);
            }
            else
            {
                return _GetListByAppId(AppId);
            }
        }

        #endregion

        #region Private Methods

        private WidgetDefinition _GetById(int id)
        {
            var query = from item in _widgetDefinitionRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        private WidgetDefinition _GetByName(string name)
        {
            var query = from item in _widgetDefinitionRepository.Table
                        where item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        private List<WidgetDefinition> _GetList()
        {
            var query = from item in _widgetDefinitionRepository.Table
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        private List<WidgetDefinition> _GetListByAppId(int id)
        {
            var query = from item in _widgetDefinitionRepository.Table
                        where item.AppId == id
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

	}

}