﻿using Weapsy.Web.Admin.Controllers;
using System;
using Weapsy.Services;
using Weapsy.Core.Caching;
using System.Web.Mvc;
using Xunit;
using Moq;
using System.Collections.Generic;
using Weapsy.Entities;
using Weapsy.Web.Admin.Extensions.Moles;
using Weapsy.Web.Admin.Models;

namespace Weapsy.Tests.Controllers.Admin
{
    public class HomeControllerTest
    {
        Mock<IAppService> mockAppService;
        Mock<CacheManager> mockCacheManager;

        public HomeControllerTest()
        {
            mockAppService = new Mock<IAppService>();
            mockCacheManager = new Mock<CacheManager>();
        }

        [Fact]
        public void ClearCache_ClearsCacheManagerAndRedirectsToIndexAction()
        {
            HomeController target = new HomeController(mockAppService.Object, mockCacheManager.Object);

            var result = target.ClearCache() as RedirectToRouteResult;

            //mockCacheManager.Verify(x => x.Clear(), Times.Once());
            Assert.Equal("Index", result.RouteValues["action"]);
        }

        [Fact]
        [Moled]
        public void Index_ReturnsViewWhereModelIsAppModelList()
        {
            HomeController target = new HomeController(mockAppService.Object, mockCacheManager.Object);
            mockAppService.Setup(x => x.GetList()).Returns(Stubs.GetAppListStub());
            MMappingExtensions.ToModelsListOfApp = (lst) => { return Stubs.GetAppModelListStub(); };

            var result = target.Index() as ViewResult;

            Assert.IsType(typeof(List<AppModel>), result.ViewData.Model);
        }

        [Fact]
        [Moled]
        public void RestartApplication_CallsUtilsMethodAndRedirectsToIndexAction()
        {
            HomeController target = new HomeController(mockAppService.Object, mockCacheManager.Object);
            bool restartWasCalled = false;
            Weapsy.Moles.MUtils.RestartApplication = () => { restartWasCalled = true; };

            var result = target.RestartApplication() as RedirectToRouteResult;

            Assert.Equal("Index", result.RouteValues["action"]);
            Assert.True(restartWasCalled);
        }
    }
}
