﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Xunit;
using Moq;
using Weapsy.Data;
using Weapsy.Entities;
using Weapsy.Core.Caching;
using Weapsy.Services;
using System.Web.Security;
using System.Web.Security.Moles;
using System.Web.Moles;
using Microsoft.Moles.Framework;
using System.Web;
using Weapsy.Moles;

namespace Weapsy.Tests.Services
{
    public class InstallServiceTest : IDisposable
    {
        private Mock<IAppService> mockAppService;
        private Mock<IEmailAccountService> mockEmailAccountService;
        private Mock<ILanguageService> mockLanguageService;
        private Mock<IMailTemplateService> mockMailTemplateService;
        private Mock<IMailTemplateTypeService> mockMailTemplateTypeService;
        private Mock<IModuleDefinitionService> mockModuleDefinitionService;
        private Mock<IModuleService> mockModuleService;
        private Mock<IModuleTemplateService> mockModuleTemplateService;
        private Mock<IPageService> mockPageService;
        private Mock<IPageTemplateService> mockPageTemplateService;
        private Mock<IPluginDefinitionService> mockPluginDefinitionService;
        private Mock<IPluginService> mockPluginService;
        private Mock<IRoleService> mockRoleService;
        private Mock<ISettingService> mockSettingService;
        private Mock<ISiteService> mockSiteService;
        private Mock<ITextService> mockTextService;
        private Mock<IThemeService> mockThemeService;
        private Mock<IUserService> mockUserService;
        private Mock<IWidgetService> mockWidgetService;
        private Mock<IWidgetDefinitionService> mockWidgetDefinitionService;
        private Mock<ICacheManager> mockCacheManager;

        public InstallServiceTest()
        {
            mockAppService = new Mock<IAppService>();
            mockEmailAccountService = new Mock<IEmailAccountService>();
            mockLanguageService = new Mock<ILanguageService>();
            mockMailTemplateService = new Mock<IMailTemplateService>();
            mockMailTemplateTypeService = new Mock<IMailTemplateTypeService>();
            mockModuleDefinitionService = new Mock<IModuleDefinitionService>();
            mockModuleService = new Mock<IModuleService>();
            mockModuleTemplateService = new Mock<IModuleTemplateService>();
            mockPageService = new Mock<IPageService>();
            mockPageTemplateService = new Mock<IPageTemplateService>();
            mockPluginDefinitionService = new Mock<IPluginDefinitionService>();
            mockPluginService = new Mock<IPluginService>();
            mockRoleService = new Mock<IRoleService>();
            mockSettingService = new Mock<ISettingService>();
            mockSiteService = new Mock<ISiteService>();
            mockTextService = new Mock<ITextService>();
            mockThemeService = new Mock<IThemeService>();
            mockUserService = new Mock<IUserService>();
            mockWidgetService = new Mock<IWidgetService>();
            mockWidgetDefinitionService = new Mock<IWidgetDefinitionService>();
            mockCacheManager = new Mock<ICacheManager>();
        }

        [Fact]
        [Moled]
        public void EnsureAppInstalled_AppDoesNotExist_CreatesNew()
        {
            InstallService target = GetTestInstallService();
            mockAppService.Setup(x => x.GetByName(It.IsAny<string>())).Returns((App)null);

            target.EnsureAppInstalled(new App());

            mockAppService.Verify(x => x.Create(It.IsAny<App>()), Times.Once());
        }

        [Fact]
        [Moled]
        public void EnsureAppInstalled_AppExists_DoesNotCreateNew()
        {
            InstallService target = GetTestInstallService();
            mockAppService.Setup(x => x.GetByName(It.IsAny<string>())).Returns(new App());

            target.EnsureAppInstalled(new App());

            mockAppService.Verify(x => x.Create(It.IsAny<App>()), Times.Never());
        }

        [Fact]
        [Moled]
        public void EnsureModuleDefinitionInstalled_ModuleDefDoesNotExist_CreatesNew()
        {
            InstallService target = GetTestInstallService();
            mockModuleDefinitionService.Setup(x => x.GetByName(It.IsAny<string>())).Returns((ModuleDefinition)null);

            target.EnsureModuleDefinitionInstalled(new ModuleDefinition());

            mockModuleDefinitionService.Verify(x => x.Create(It.IsAny<ModuleDefinition>()), Times.Once());
        }

        [Fact]
        [Moled]
        public void EnsureModuleDefinitionInstalled_ModuleDefExists_DoesNotCreateNew()
        {
            InstallService target = GetTestInstallService();
            mockModuleDefinitionService.Setup(x => x.GetByName(It.IsAny<string>())).Returns(new ModuleDefinition());

            target.EnsureModuleDefinitionInstalled(new ModuleDefinition());

            mockModuleDefinitionService.Verify(x => x.Create(It.IsAny<ModuleDefinition>()), Times.Never());
        }

        [Fact]
        [Moled]
        public void EnsurePluginDefinitionInstalled_PluginDefDoesNotExist_CreatesNew()
        {
            InstallService target = GetTestInstallService();
            mockPluginDefinitionService.Setup(x => x.GetByName(It.IsAny<string>())).Returns((PluginDefinition)null);

            target.EnsurePluginDefinitionInstalled(new PluginDefinition());

            mockPluginDefinitionService.Verify(x => x.Create(It.IsAny<PluginDefinition>()), Times.Once());
        }

        [Fact]
        [Moled]
        public void EnsurePluginDefinitionInstalled_PluginDefExists_DoesNotCreateNew()
        {
            InstallService target = GetTestInstallService();
            mockPluginDefinitionService.Setup(x => x.GetByName(It.IsAny<string>())).Returns(new PluginDefinition());

            target.EnsurePluginDefinitionInstalled(new PluginDefinition());

            mockPluginDefinitionService.Verify(x => x.Create(It.IsAny<PluginDefinition>()), Times.Never());
        }

        [Fact]
        [Moled]
        public void EnsureWidgetDefinitionInstalled_WidgetDefDoesNotExist_CreatesNew()
        {
            InstallService target = GetTestInstallService();
            mockWidgetDefinitionService.Setup(x => x.GetByName(It.IsAny<string>())).Returns((WidgetDefinition)null);

            target.EnsureWidgetDefinitionInstalled(new WidgetDefinition());

            mockWidgetDefinitionService.Verify(x => x.Create(It.IsAny<WidgetDefinition>()), Times.Once());
        }

        [Fact]
        [Moled]
        public void EnsureWidgetDefinitionInstalled_WidgetDefExists_DoesNotCreateNew()
        {
            InstallService target = GetTestInstallService();
            mockWidgetDefinitionService.Setup(x => x.GetByName(It.IsAny<string>())).Returns(new WidgetDefinition());

            target.EnsureWidgetDefinitionInstalled(new WidgetDefinition());

            mockWidgetDefinitionService.Verify(x => x.Create(It.IsAny<WidgetDefinition>()), Times.Never());
        }

        [Fact]
        public void Initialize()
        {
            InstallService target = GetTestInstallService();
            mockSiteService.Setup(x => x.GetById(It.IsAny<int>())).Returns(new Site());
            bool restartWasCalled = false;

            using (MolesContext.Create())
            {
                MMembership.GetUserString = (name) => { return new Mock<MembershipUser>().Object; };
                MRoles.RoleExistsString = (name) => { return true; };
                MRoles.IsUserInRoleStringString = (name, role) => { return true; };
                MHttpUtility.UrlEncodeString = (url) => { return "weapsy.com"; };
                MUtils.RestartApplication = () => { restartWasCalled = true; };

                var context = MvcMockHelpers.FakeHttpContext();
                MHttpContext.CurrentGet = () => { return context; };

                target.Initialize();
            }

            Assert.True(restartWasCalled);
        }

        #region Private Methods
        private InstallService GetTestInstallService()
        {
            return new InstallService(mockAppService.Object, mockEmailAccountService.Object,
                mockLanguageService.Object, mockMailTemplateService.Object, mockMailTemplateTypeService.Object,
                mockModuleDefinitionService.Object, mockModuleService.Object, mockModuleTemplateService.Object,
                mockPageService.Object, mockPageTemplateService.Object, mockPluginDefinitionService.Object,
                mockPluginService.Object, mockRoleService.Object, mockSettingService.Object,
                mockSiteService.Object, mockTextService.Object, mockThemeService.Object,
                mockUserService.Object, mockWidgetService.Object, mockWidgetDefinitionService.Object,
                mockCacheManager.Object);
        }
        #endregion Private Methods

        public void Dispose()
        {
            // dispose
        }
    }
}
