﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Weapsy.Entities;
using Weapsy.Web.Admin.Models;
using System.Web.Security;
using Moq;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Collections.Specialized;
using System.Web.Moles;

namespace Weapsy.Tests
{
    public class Stubs
    {
        public static List<App> GetAppListStub()
        {
            List<App> appList = new List<App>()
            {
                new App { Id=1, Name="TestApp1" },
                new App { Id=2, Name="TestApp2" },
                new App { Id=3, Name="TestApp3" }
            };

            return appList;
        }

        public static List<AppModel> GetAppModelListStub()
        {
            List<AppModel> appModelList = new List<AppModel>()
            {
                new AppModel { Id=1, Name="TestApp1" },
                new AppModel { Id=2, Name="TestApp2" },
                new AppModel { Id=3, Name="TestApp3" }
            };

            return appModelList;
        }

        public static List<EmailAccount> GetEmailAccountListStub()
        {
            List<EmailAccount> emailAccountList = new List<EmailAccount>()
            {
                new EmailAccount { Id=1, Address="test1@test.com", SiteId=1 },
                new EmailAccount { Id=2, Address="test2@test.com", SiteId=1 },
                new EmailAccount { Id=3, Address="test3@test.com", SiteId=1 }
            };

            return emailAccountList;
        }

        public static List<ModuleDefinition> GetModuleDefinitionListStub()
        {
            List<ModuleDefinition> moduleDefinitionList = new List<ModuleDefinition>()
            {
                new ModuleDefinition { Id=1, AppId=1, Name="TestModuleDefinition1" },
                new ModuleDefinition { Id=2, AppId=1, Name="TestModuleDefinition2" },
                new ModuleDefinition { Id=3, AppId=1, Name="TestModuleDefinition3" },
            };

            return moduleDefinitionList;
        }

        public static List<PluginDefinition> GetPluginDefinitionListStub()
        {
            List<PluginDefinition> pluginDefinitionList = new List<PluginDefinition>()
            {
                new PluginDefinition { Id=1, AppId=1, Name="TestPluginDefinition1" },
                new PluginDefinition { Id=2, AppId=1, Name="TestPluginDefinition2" },
                new PluginDefinition { Id=3, AppId=1, Name="TestPluginDefinition3" }
            };

            return pluginDefinitionList;
        }

        public static List<WidgetDefinition> GetWidgetDefinitionListStub()
        {
            List<WidgetDefinition> widgetDefinitionList = new List<WidgetDefinition>()
            {
                new WidgetDefinition { Id=2, AppId=1, Name="TestWidgetDefinition2" },
                new WidgetDefinition { Id=1, AppId=1, Name="TestWidgetDefinition1" },
                new WidgetDefinition { Id=3, AppId=1, Name="TestWidgetDefinition3" }
            };

            return widgetDefinitionList;
        }

        public static List<Language> GetLanguageListStub()
        {
            List<Language> languageList = new List<Language>()
            {
                new Language { Id = 1, SiteId = 1, Name = "English", CultureName = "en", Url = "weapsy.com/en", SortOrder = 1 },
                new Language { Id = 2, SiteId = 1, Name = "Spanish", CultureName = "es", Url = "weapsy.com/es", SortOrder = 2 },
                new Language { Id = 3, SiteId = 1, Name = "Italian", CultureName = "it", Url = "weapsy.com/it", SortOrder = 3 }
            };

            return languageList;
        }

        public static AppModel GetFakeAppModel()
        {
            AppModel appModel = new AppModel()
            {
                Id = 1,
                Name = "TestApp1",
                Title = "TestApp1",
                Folder = "TestAppFolder",
                Url = "TestUrl",
                AdminController = "TestController",
            };

            return appModel;
        }

        public static MembershipUser GetFakeMembershipUser(string name)
        {
            return new Mock<MembershipUser>().Object;
        }
    }

    public static class MvcMockHelpers
    {
        public static MHttpContext FakeHttpContext()
        {
            var context = new MHttpContext();
            var request = new MHttpRequest();
            var response = new MHttpResponse();

            request.UserHostAddressGet = () => "192.168.0.0";
            context.RequestGet = () => request;
            context.ResponseGet = () => response;            

            return context;
        }

        //public static HttpContextBase FakeHttpContext(string url)
        //{
        //    HttpContextBase context = FakeHttpContext();
        //    context.Request.SetupRequestUrl(url);
        //    return context;
        //}

        //public static void SetFakeControllerContext(this Controller controller)
        //{
        //    var httpContext = FakeHttpContext();
        //    ControllerContext context = new ControllerContext(new RequestContext(httpContext, new RouteData()), controller);
        //    controller.ControllerContext = context;
        //}

        static string GetUrlFileName(string url)
        {
            if (url.Contains("?"))
                return url.Substring(0, url.IndexOf("?"));
            else
                return url;
        }

        static NameValueCollection GetQueryStringParameters(string url)
        {
            if (url.Contains("?"))
            {
                NameValueCollection parameters = new NameValueCollection();

                string[] parts = url.Split("?".ToCharArray());
                string[] keys = parts[1].Split("&".ToCharArray());

                foreach (string key in keys)
                {
                    string[] part = key.Split("=".ToCharArray());
                    parameters.Add(part[0], part[1]);
                }

                return parameters;
            }
            else
            {
                return null;
            }
        }

        public static void SetHttpMethodResult(this HttpRequestBase request, string httpMethod)
        {
            Mock.Get(request)
                .Setup(req => req.HttpMethod)
                .Returns(httpMethod);
        }

        public static void SetupRequestUrl(this HttpRequestBase request, string url)
        {
            if (url == null)
                throw new ArgumentNullException("url");

            if (!url.StartsWith("~/"))
                throw new ArgumentException("Sorry, we expect a virtual url starting with \"~/\".");

            var mock = Mock.Get(request);

            mock.Setup(req => req.QueryString)
                .Returns(GetQueryStringParameters(url));
            mock.Setup(req => req.AppRelativeCurrentExecutionFilePath)
                .Returns(GetUrlFileName(url));
            mock.Setup(req => req.PathInfo)
                .Returns(string.Empty);
        }
    }
}
