#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class AppController : BaseAdminController
    {

        #region Private Members

        private readonly IAppService _appService;
        private readonly IModuleDefinitionService _moduleDefinitionService;
        private readonly IPluginDefinitionService _pluginDefinitionService;
        private readonly IWidgetDefinitionService _widgetDefinitionService;

        #endregion

        #region Constructors

        public AppController(IAppService appService,
            IModuleDefinitionService moduleDefinitionService,
            IPluginDefinitionService pluginDefinitionService,
            IWidgetDefinitionService widgetDefinitionService)
        {
            _appService = appService;
            _moduleDefinitionService = moduleDefinitionService;
            _pluginDefinitionService = pluginDefinitionService;
            _widgetDefinitionService = widgetDefinitionService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            return View(_appService.GetList().ToModels());
        }

        public ActionResult Create()
        {
            return View(new AppModel());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(AppModel model)
        {
            if (ModelState.IsValid)
            {
                if (_appService.GetByName(model.Name, false) != null)
                { 
                    ModelState.AddModelError("", "An App with the same Name already exists");
                    return View(model);
                }

                App entity = model.ToEntity();
                entity.SetAuditForCreate();
                _appService.Create(entity);

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            App entity = _appService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "App Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(AppModel model)
        {
            App entity = _appService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "App Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_appService.GetByName(model.Name, false) != null)
                    {
                        //if (TempData["Error"] == null) TempData["Error"] = new List<string>();
                        //((List<string>)TempData["Error"]).Add("An App with the same Name already exists");
                        return RedirectToAction("Edit", new { id = entity.Id });
                    }
                }

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _appService.Update(entity);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(AppModel model)
        {
            App entity = _appService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "App Not Found");
            }

            if (ModelState.IsValid)
            {
                foreach (var item in _moduleDefinitionService.GetListByAppId(entity.Id))
                {
                    _moduleDefinitionService.Delete(item);
                }

                foreach (var item in _pluginDefinitionService.GetListByAppId(entity.Id))
                {
                    _pluginDefinitionService.Delete(item);
                }

                foreach (var item in _widgetDefinitionService.GetListByAppId(entity.Id))
                {
                    _widgetDefinitionService.Delete(item);
                }

                _appService.Delete(entity);

                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}