#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class EmailAccountController : BaseAdminController
    {

        #region Private Members

        private readonly IEmailAccountService _emailAccountService;

        #endregion

        #region Constructors

        public EmailAccountController(IEmailAccountService emailAccountService)
        {
            _emailAccountService = emailAccountService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            return View(_emailAccountService.GetList(SiteId).ToModels());
        }

        public ActionResult Create()
        {
            return View(new EmailAccountModel());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(EmailAccountModel model)
        {
            if (ModelState.IsValid)
            {
                EmailAccount entity = model.ToEntity();
                entity.SiteId = SiteId;
                entity.SetAuditForCreate();

                _emailAccountService.Create(entity);

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            EmailAccount entity = _emailAccountService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Email Account Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(EmailAccountModel model)
        {
            EmailAccount entity = _emailAccountService.GetById(SiteId, model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Email Account Not Found");
            }

            if (ModelState.IsValid)
            {
                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                _emailAccountService.Update(entity);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(EmailAccountModel model)
        {
            EmailAccount entity = _emailAccountService.GetById(SiteId, model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Email Account Not Found");
            }

            if (ModelState.IsValid)
            {
                _emailAccountService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion
    
    }

}