#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class LanguageController : BaseAdminController
    {

        #region Private Members

        private readonly ILanguageService _languageService;

        #endregion

        #region Constructors

        public LanguageController(ILanguageService languageService)
        {
            _languageService = languageService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            return View(new List<LanguageModel>());
        }

        public PartialViewResult Grid(int id, string action)
        {
            if (id > 0 && action != "")
            {
                _languageService.Move(SiteId, id, action);
            }

            return PartialView("_Grid", _languageService.GetList(SiteId).ToModels());
        }

        public ActionResult Create()
        {
            LanguageModel model = new LanguageModel();

            model.SortOrder = _languageService.GetList(SiteId).Count + 1;

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(LanguageModel model)
        {
            if (ModelState.IsValid)
            {
                if (_languageService.GetByName(SiteId, model.Name, false) != null)
                {
                    ModelState.AddModelError("", "A Language with the same Name already exists");
                    return View(model);
                }

                Language entity = model.ToEntity();
                entity.SiteId = SiteId;
                entity.SetAuditForCreate();

                CultureInfo info = CultureInfo.GetCultureInfo(model.CultureName);

                entity.CultureNativeName = info.NativeName;
                entity.CultureEnglishName = info.EnglishName;

                _languageService.Create(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            Language entity = _languageService.GetById(SiteId, id);

            if (entity == null)
            {
                throw new HttpException(404, "Language Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(LanguageModel model)
        {
            Language entity = _languageService.GetById(SiteId, model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Language Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_languageService.GetByName(SiteId, model.Name, false) != null)
                    {
                        return RedirectToAction("Edit", new { id = entity.Id });
                    }
                }

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                CultureInfo info = CultureInfo.GetCultureInfo(model.CultureName);

                entity.CultureNativeName = info.NativeName;
                entity.CultureEnglishName = info.EnglishName;

                _languageService.Update(entity);

                return RedirectToAction("List"); 
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(LanguageModel model)
        {
            Language entity = _languageService.GetById(SiteId, model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Language Not Found");
            }

            if (ModelState.IsValid)
            {
                _languageService.Delete(entity);
                return RedirectToAction("List");                
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}