#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class MailTemplateTypeController : BaseAdminController
    {

        #region Private Members

        private readonly IAppService _appService;
        private readonly ILanguageService _languageService;
        private readonly IMailTemplateTypeService _mailTemplateTypeService;

        #endregion

        #region Constructors

        public MailTemplateTypeController(IAppService appService,
            ILanguageService languageService,
            IMailTemplateTypeService mailTemplateTypeService)
        {
            _appService = appService;
            _languageService = languageService;
            _mailTemplateTypeService = mailTemplateTypeService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            return View(_mailTemplateTypeService.GetList().ToModels());
        }

        public ActionResult Create()
        {
            return View(new MailTemplateTypeModel());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(MailTemplateTypeModel model)
        {
            if (ModelState.IsValid)
            {
                MailTemplateType entity = model.ToEntity();
                entity.SetAuditForCreate();

                _mailTemplateTypeService.Create(entity);

                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            MailTemplateType entity = _mailTemplateTypeService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "Mail Template Type Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(MailTemplateTypeModel model)
        {
            MailTemplateType entity = _mailTemplateTypeService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Mail Template Type Not Found");
            }

            if (ModelState.IsValid)
            {
                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();

                _mailTemplateTypeService.Update(entity);

                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(MailTemplateTypeModel model)
        {
            MailTemplateType entity = _mailTemplateTypeService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Mail Template Type Not Found");
            }

            if (ModelState.IsValid)
            {
                _mailTemplateTypeService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        public PartialViewResult ChangePasswordTokens()
        {
            return PartialView("_ChangePasswordTokens");
        }

        public PartialViewResult PasswordRecoveryTokens()
        {
            return PartialView("_PasswordRecoveryTokens");
        }

        public PartialViewResult RegisterTokens()
        {
            return PartialView("_RegisterTokens");
        }

        #endregion
    
    }

}