#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class ModuleDefinitionController : BaseAdminController
    {

        #region Private Members

        private readonly IAppService _appService;
        private readonly IModuleDefinitionService _moduleDefinitionService;

        #endregion

        #region Constructors

        public ModuleDefinitionController(IAppService appService, IModuleDefinitionService moduleDefinitionService)
        {
            _appService = appService;
            _moduleDefinitionService = moduleDefinitionService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List(int id = -2)
        {
            ViewBag.AppId = id;

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("-2", "<All>");
            dic.Add("-1", "<Default>");
            foreach (var item in _appService.GetList())
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }

            return View(dic);
        }

        public PartialViewResult Grid(int appId)
        {
            List<ModuleDefinitionModel> model = new List<ModuleDefinitionModel>();

            if (appId > 0)
            {
                model = _moduleDefinitionService.GetListByAppId(appId).ToModels();
            }
            else
            {
                if (appId == -1)
                {
                    model = _moduleDefinitionService.GetListByAppId(-1).ToModels();
                }
                else
                {
                    model = _moduleDefinitionService.GetList().ToModels();
                }
            }
            
            foreach (var item in model)
            {
                if (item.AppId > 0)
                {
                    item.AppTitle = _appService.GetById(item.AppId).Title;
                }
                else
                {
                    item.AppTitle = "Default";
                }
            }

            return PartialView("_Grid", model);
        }

        public ActionResult Create(int id = 0)
        {
            ViewBag.AppId = id;
            ModuleDefinitionModel model = new ModuleDefinitionModel();
            model.Apps = _appService.GetDictionary();
            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(ModuleDefinitionModel model)
        {
            if (ModelState.IsValid)
            {
                if (_moduleDefinitionService.GetByName(model.Name, false) != null)
                {
                    ModelState.AddModelError("", "A Module Definition with the same Name already exists");
                    return View(model);
                }

                ModuleDefinition entity = model.ToEntity();
                entity.SetAuditForCreate();
                _moduleDefinitionService.Create(entity);
                return RedirectToAction("List");                
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            ModuleDefinition entity = _moduleDefinitionService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "Module Definition Not Found");
            }

            ModuleDefinitionModel model = new ModuleDefinitionModel();

            model = entity.ToModel();
            model.Apps = _appService.GetDictionary();

            if (model.AppId > 0)
            {
                model.AppTitle = _appService.GetById(model.AppId).Title;
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(ModuleDefinitionModel model)
        {
            ModuleDefinition entity = _moduleDefinitionService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Module Definition Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_moduleDefinitionService.GetByName(model.Name, false) != null)
                    {
                        return RedirectToAction("Edit", new { id = entity.Id });
                    }
                }

                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _moduleDefinitionService.Update(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(ModuleDefinitionModel model)
        {
            ModuleDefinition entity = _moduleDefinitionService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Module Definition Not Found");
            }

            if (ModelState.IsValid)
            {
                _moduleDefinitionService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}