#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class PageTemplateController : BaseAdminController
    {

        #region Private Members

        private readonly IPageTemplateService _pageTemplateService;

        #endregion

        #region Constructors

        public PageTemplateController(IPageTemplateService pageTemplateService)
        {
            _pageTemplateService = pageTemplateService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            return View(new List<PageTemplateModel>());
        }

        public PartialViewResult Grid(int id, string action)
        {
            if (id > 0 && action != "")
            {
                //BusinessLogic.PageTemplates.Move(id, action);
            }
            return PartialView("_Grid", _pageTemplateService.GetList().ToModels());
        }

        public ActionResult Create()
        {
            return View(new PageTemplateModel());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(PageTemplateModel model)
        {
            if (ModelState.IsValid)
            {
                PageTemplate entity = model.ToEntity();
                entity.SetAuditForCreate();
                _pageTemplateService.Create(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            PageTemplate entity = _pageTemplateService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "Page Template Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(PageTemplateModel model)
        {
            PageTemplate entity = _pageTemplateService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Page Template Not Found");
            }

            if (ModelState.IsValid)
            {
                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _pageTemplateService.Update(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(PageTemplateModel model)
        {
            PageTemplate entity = _pageTemplateService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Page Template Not Found");
            }

            if (ModelState.IsValid)
            {
                _pageTemplateService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}