#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class RoleController : BaseAdminController
    {

        #region Private Members

        private readonly IRoleService _roleService;

        #endregion

        #region Constructors

        public RoleController(IRoleService roleService)
        {
            _roleService = roleService;
        }

        #endregion

        #region Public Methods

        public ActionResult List()
        {
            return View();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(string RoleToCreate)
        {
            if (!string.IsNullOrEmpty(RoleToCreate))
            {
                if (!System.Web.Security.Roles.RoleExists(RoleToCreate))
                {
                    System.Web.Security.Roles.CreateRole(RoleToCreate);
                    if (Request.IsAjaxRequest())
                    {
                        return Content("Role created!");
                    }
                    else
                    {
                        return RedirectToAction("List");
                    }
                }
                else
                {
                    ModelState.AddModelError("", "Role " + RoleToCreate + " already exists.");
                }
            }

            return View();
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(string RoleToDelete)
        {
            if (!string.IsNullOrEmpty(RoleToDelete))
            {
                if (System.Web.Security.Roles.GetUsersInRole(RoleToDelete).Length == 0)
                {
                    System.Web.Security.Roles.DeleteRole(RoleToDelete);
                    if (Request.IsAjaxRequest())
                    {
                        return Content("Role deleted!");
                    }
                    else
                    {
                        return RedirectToAction("List");
                    }
                }
                else
                {
                    ModelState.AddModelError("", "There are Users in Role " + RoleToDelete + ".");
                }
            }

            return View();
        }

        #endregion

    }

}