#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class SettingController : BaseAdminController
    {

        #region Private Members

        private readonly ISettingService _settingService;

        #endregion

        #region Constructors

        public SettingController(ISettingService settingService)
        {
            _settingService = settingService;
        }

        #endregion

        #region Public Methods

        public ActionResult Settings()
        {
            SettingsModel model = new SettingsModel();

            model.MemberSettings = _settingService.GetSettings<MemberSettings>().ToModel();
            model.SecuritySettings = _settingService.GetSettings<SecuritySettings>().ToModel();

            return View(model);
        }

        [HttpPost]
        public ActionResult Settings(SettingsModel model)
        {
            if (ModelState.IsValid)
            {
                _settingService.Save<MemberSettings>(model.MemberSettings.ToEntity());
                _settingService.Save<SecuritySettings>(model.SecuritySettings.ToEntity());

                TempData[Consts.MessageKeySuccess] = Resources.Admin.Setting.Settings_ConfirmUpdate;

                return RedirectToAction("Index", "Home");
            }

            return View(model);
        }

        public ActionResult MemberSettings()
        {
            SettingsModel model = new SettingsModel();

            model.MemberSettings = _settingService.GetSettings<MemberSettings>().ToModel();

            return View(model);
        }

        [HttpPost]
        public ActionResult MemberSettings(SettingsModel model)
        {
            if (ModelState.IsValid)
            {
                _settingService.Save<MemberSettings>(model.MemberSettings.ToEntity());

                return RedirectToAction("MemberSettings");
            }

            return View(model);
        }

        public ActionResult SecuritySettings()
        {
            SettingsModel model = new SettingsModel();

            model.SecuritySettings = _settingService.GetSettings<SecuritySettings>().ToModel();

            return View(model);
        }

        [HttpPost]
        public ActionResult SecuritySettings(SettingsModel model)
        {
            if (ModelState.IsValid)
            {
                _settingService.Save<SecuritySettings>(model.SecuritySettings.ToEntity());

                return RedirectToAction("SecuritySettings");
            }

            return View(model);
        }

        public ActionResult List()
        {
            return View(_settingService.GetList().ToModels());
        }

        public ActionResult Create()
        {
            return View(new SettingModel());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(SettingModel model)
        {
            if (ModelState.IsValid)
            {
                if (_settingService.GetByName(model.Name, false) != null)
                {
                    ModelState.AddModelError("", "A Setting with the same Name already exists");
                    return View(model);
                }
                Setting entity = model.ToEntity();
                _settingService.Create(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            Setting entity = _settingService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "Setting Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(SettingModel model)
        {
            Setting entity = _settingService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Setting Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_settingService.GetByName(model.Name, false) != null)
                    {
                        return RedirectToAction("Edit", new { id = entity.Id });
                    }
                }

                entity = model.ToEntity(entity);
                _settingService.Update(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(SettingModel model)
        {
            Setting entity = _settingService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Setting Not Found");
            }

            if (ModelState.IsValid)
            {
                _settingService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}