#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class ThemeController : BaseAdminController
    {

        #region Private Members

        private readonly IThemeService _themeService;

        #endregion

        #region Constructors

        public ThemeController(IThemeService themeService)
        {
            _themeService = themeService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List()
        {
            return View(new List<ThemeModel>());
        }

        public PartialViewResult Grid(int id, string action)
        {
            if (id > 0 && action != "")
            {
                _themeService.Move(id, action);
            }
            return PartialView("_Grid", _themeService.GetList().ToModels());
        }

        public ActionResult Create()
        {
            ThemeModel model = new ThemeModel();

            model.SortOrder = _themeService.GetList().Count + 1;

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(ThemeModel model)
        {
            if (ModelState.IsValid)
            {
                if (_themeService.GetByName(model.Name) != null)
                {
                    ModelState.AddModelError("", "A Theme with the same Name already exists");
                    return View(model);
                }
                Theme entity = model.ToEntity();
                entity.SetAuditForCreate();
                _themeService.Create(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            Theme entity = _themeService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "Theme Not Found");
            }

            return View(entity.ToModel());
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(ThemeModel model)
        {
            Theme entity = _themeService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Theme Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_themeService.GetByName(model.Name) != null)
                    {
                        return RedirectToAction("Edit", new { id = entity.Id });
                    }
                }
                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _themeService.Update(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(ThemeModel model)
        {
            Theme entity = _themeService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Theme Not Found");
            }

            if (ModelState.IsValid)
            {
                _themeService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}