#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Data;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;
using Telerik.Web.Mvc;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class UserController : BaseAdminController
    {

        #region Private Members

        private readonly IUserService _userService;

        #endregion

        #region Constructors

        public UserController(IUserService userService)
        {
            _userService = userService;
        }

        #endregion

        #region Public Methods

        public ActionResult List()
        {
            return View();
        }

        [GridAction]
        public ActionResult Grid()
        {
            return View(new GridModel(Users()));
        }

        private List<UserModel> Users()
        {
            List<UserModel> model = _userService.GetList().ToModels();

            foreach (var item in model)
            {
                string roles = "";
                foreach (string role in Roles.GetRolesForUser(item.MembershipName))
                {
                    if (roles == "")
                    {
                        roles += role;
                    }
                    else
                    {
                        roles += "," + role;
                    }
                }
                item.Roles = roles;
            }

            return model;
        }

        public ActionResult Create()
        {
            return View(new UserModel());
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(UserModel model)
        {
            if (ModelState.IsValid)
            {
                MembershipCreateStatus createStatus;
                Membership.CreateUser(model.MembershipName, Membership.GeneratePassword(Membership.MinRequiredPasswordLength, Membership.MinRequiredNonAlphanumericCharacters), model.MembershipEmail, null, null, true, null, out createStatus);

                if (createStatus == MembershipCreateStatus.Success)
                {
                    User entity = model.ToEntity();

                    entity.SetAuditForCreate();

                    entity.ProfileName = Utils.ParseTextForUrl(model.MembershipName);
                    entity.MembershipDate = Membership.GetUser(model.MembershipName).CreationDate;

                    _userService.Create(entity);

                    foreach (var role in Roles.GetAllRoles())
                    {
                        if (Request.Form["Role-" + role].Contains("true"))
                        {
                            if (!Roles.IsUserInRole(entity.MembershipName, role))
                            {
                                Roles.AddUserToRole(entity.MembershipName, role);
                            }
                        }
                        else
                        {
                            if (Roles.IsUserInRole(entity.MembershipName, role))
                            {
                                Roles.RemoveUserFromRole(entity.MembershipName, role);
                            }
                        }
                    }

                    return RedirectToAction("List");
                }
                else
                {
                    ModelState.AddModelError("", createStatus.ToString());
                    return View(model);
                }
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            User entity = _userService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "User Not Found");
            }

            UserModel model = entity.ToModel();

            MembershipUser mUser = Membership.GetUser(entity.MembershipName);

            model.IsApproved = mUser.IsApproved;
            model.Comment = mUser.Comment;
            model.IsLockedOut = mUser.IsLockedOut;
            model.LastLockoutDate = mUser.LastLockoutDate;

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(UserModel model)
        {
            User entity = _userService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "User Not Found");
            }

            if (ModelState.IsValid)
            {
                MembershipUser mUser = Membership.GetUser(entity.MembershipName);

                entity = model.ToEntity(entity);

                //if (string.IsNullOrWhiteSpace(model.MembershipName))
                //{
                    entity.MembershipName = mUser.UserName;
                //}

                entity.SetAuditForUpdate();

                _userService.Update(entity);

                //if (!string.IsNullOrWhiteSpace(model.MembershipName))
                //{
                //    if (model.MembershipName != mUser.UserName)
                //    {
                //        AspNetUser aspnetUser = _aspNetUserService.GetByUserName(mUser.UserName, Membership.Provider.ApplicationName);

                //        aspnetUser.UserName = model.MembershipName;
                //        aspnetUser.LoweredUserName = model.MembershipName.ToLower();

                //        _aspNetUserService.Update(aspnetUser);
                //    }
                //}
                
                mUser.Email = model.MembershipEmail;
                mUser.Comment = model.Comment;
                mUser.IsApproved = model.IsApproved;

                Membership.UpdateUser(mUser);

                foreach (var role in Roles.GetAllRoles())
                {
                    if (Request.Form["Role-" + role].Contains("true"))
                    {
                        if (!Roles.IsUserInRole(entity.MembershipName, role))
                        {
                            Roles.AddUserToRole(entity.MembershipName, role);
                        }
                    }
                    else
                    {
                        if (Roles.IsUserInRole(entity.MembershipName, role))
                        {
                            Roles.RemoveUserFromRole(entity.MembershipName, role);
                        }
                    }
                }

                return RedirectToAction("List");
            }

            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Unlock(UserModel model)
        {
            User entity = _userService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "User Not Found");
            }

            if (ModelState.IsValid)
            {
                MembershipUser User = Membership.GetUser(model.MembershipName);

                if (User != null)
                {
                    User.UnlockUser();
                }

                return RedirectToAction("Edit", new { id = model.Id });
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(UserModel model)
        {
            User entity = _userService.GetById(model.Id);

            if (entity == null)
            {
                throw new HttpException(404, "User Not Found");
            }

            if (ModelState.IsValid)
            {
                _userService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}