#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using Weapsy.Core;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Extensions;
using Weapsy.Web.Admin.Models;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using AutoMapper;

namespace Weapsy.Web.Admin.Controllers
{

    [Authorize(Roles="SuperAdmin")]
	public class WidgetDefinitionController : BaseAdminController
    {

        #region Private Members

        private readonly IAppService _appService;
        private readonly IWidgetDefinitionService _widgetDefinitionService;

        #endregion

        #region Constructors

        public WidgetDefinitionController(IAppService appService, 
            IWidgetDefinitionService widgetDefinitionService)
        {
            _appService = appService;
            _widgetDefinitionService = widgetDefinitionService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            return RedirectToAction("List");
        }

        public ActionResult List(int id = -2)
        {
            ViewBag.AppId = id;

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("-2", "<All>");
            dic.Add("-1", "<Default>");
            foreach (var item in _appService.GetList())
            {
                dic.Add(item.Id.ToString(), item.Name.ToString());
            }

            return View(dic);
        }

        public PartialViewResult Grid(int appId)
        {
            List<WidgetDefinitionModel> model = new List<WidgetDefinitionModel>();

            if (appId > 0)
            {
                model = _widgetDefinitionService.GetListByAppId(appId).ToModels();
            }
            else
            {
                if (appId == -1)
                {
                    model = _widgetDefinitionService.GetListByAppId(-1).ToModels();
                }
                else
                {
                    model = _widgetDefinitionService.GetList().ToModels();
                }
            }
            
            foreach (var item in model)
            {
                if (item.AppId > 0)
                {
                    item.AppTitle = _appService.GetById(item.AppId).Title;
                }
                else
                {
                    item.AppTitle = "Default";
                }

                item.TypeName = ((WidgetType)item.TypeId).ToString();
            }

            return PartialView("_Grid", model);
        }

        public ActionResult Create(int id = 0)
        {
            ViewBag.AppId = id;
            WidgetDefinitionModel model = new WidgetDefinitionModel();
            model.Apps = _appService.GetDictionary();
            return View(model);
        }

        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Create(WidgetDefinitionModel model)
        {
            if (ModelState.IsValid)
            {
                if (_widgetDefinitionService.GetByName(model.Name) != null)
                {
                    ModelState.AddModelError("", "A Widget Definition with the same Name already exists");
                    return View(model);
                }
                WidgetDefinition entity = model.ToEntity();
                entity.SetAuditForCreate();
                _widgetDefinitionService.Create(entity);
                return RedirectToAction("List");                
            }

            return View(model);
        }

        public ActionResult Edit(int id)
        {
            WidgetDefinition entity = _widgetDefinitionService.GetById(id);

            if (entity == null)
            {
                throw new HttpException(404, "Widget Definition Not Found");
            }

            WidgetDefinitionModel model = new WidgetDefinitionModel();

            model = entity.ToModel();
            model.TypeName = ((WidgetType)model.TypeId).ToString();
            model.Apps = _appService.GetDictionary();

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(WidgetDefinitionModel model)
        {
            WidgetDefinition entity = _widgetDefinitionService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Widget Definition Not Found");
            }

            if (ModelState.IsValid)
            {
                if (entity.Name != model.Name)
                {
                    if (_widgetDefinitionService.GetByName(model.Name) != null)
                    {
                        return RedirectToAction("Edit", new { id = entity.Id });
                    }
                }
                entity = model.ToEntity(entity);
                entity.SetAuditForUpdate();
                _widgetDefinitionService.Update(entity);
                return RedirectToAction("List");
            }

            return View(model);
        }

        [HttpParamAction]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Delete(WidgetDefinitionModel model)
        {
            WidgetDefinition entity = _widgetDefinitionService.GetById(model.Id, false);

            if (entity == null)
            {
                throw new HttpException(404, "Widget Definition Not Found");
            }

            if (ModelState.IsValid)
            {
                _widgetDefinitionService.Delete(entity);
                return RedirectToAction("List");
            }

            return RedirectToAction("Edit", new { id = model.Id });
        }

        #endregion

    }

}