﻿#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System.Web.Mvc;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Framework;

namespace Weapsy.Apps.Forum
{
    public class ForumAreaRegistration : AreaRegistration
    {

        public ForumAreaRegistration() {
            var _installService = DependencyResolver.Current.GetService<IInstallService>();

            App = _installService.EnsureAppInstalled(new Weapsy.Entities.App()
            { 
                Name = AreaName,
                Description = "Forum App",
                Title = AreaName,
                Folder = AreaName,
                Url = AreaName.ToLower(),
                AdminController = "Admin",
                AdminAction = "Index",
                DashboardIcon = "dashboard-icon.png"
            });

            //Weapsy.Apps.Forum.Install.EnsureInstalled();
        }

        public override string AreaName
        {
            get
            {
                return "Forum";
            }
        }

        public string AreaUrl
        {
            get
            {
                return App.Url;
            }
        }

        public string AreaNamespace
        {
            get
            {
                return "Weapsy.Apps.Forum.Controllers";
            }
        }

        public Weapsy.Entities.App App { get; set; }

        public override void RegisterArea(AreaRegistrationContext context)
        {
            context.MapRoute(
                AreaName + "_Admin",
                AreaUrl + "/Admin/{action}/{id}",
                new { controller = "Admin", action = "Index", id = UrlParameter.Optional },
                null,
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_Account",
                AreaUrl + "/Account/{action}/{id}",
                new { controller = "Account", action = "Index", id = UrlParameter.Optional },
                null,
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_Module",
                AreaUrl + "/Module/{action}/{id}",
                new { controller = "Module", action = "Index", id = UrlParameter.Optional },
                null,
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_NewTopic",
                AreaUrl + "/{CategoryUrl}/{SectionUrl}/new_topic",
                new { controller = "Home", action = "CreateTopic" },
                new { CategoryUrl = @"[a-zA-Z0-9\-]*", SectionUrl = @"[a-zA-Z0-9\-]*" },
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_NewReply",
                AreaUrl + "/{CategoryUrl}/{SectionUrl}/{TopicId}_{TopicUrl}/new_reply",
                new { controller = "Home", action = "CreateReply" },
                new { CategoryUrl = @"[a-zA-Z0-9\-]*", SectionUrl = @"[a-zA-Z0-9\-]*", TopicId = @"[0-9]*", TopicUrl = @"[a-zA-Z0-9\-]*" },
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_TopicPaged",
                AreaUrl + "/{CategoryUrl}/{SectionUrl}/{TopicId}_{TopicUrl}/{PageIndex}",
                new { controller = "Home", action = "Topic" },
                new { CategoryUrl = @"[a-zA-Z0-9\-]*", SectionUrl = @"[a-zA-Z0-9\-]*", TopicId = @"[0-9]*", TopicUrl = @"[a-zA-Z0-9\-]*", PageIndex = "[0-9]*" },
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_Topic",
                AreaUrl + "/{CategoryUrl}/{SectionUrl}/{TopicId}_{TopicUrl}",
                new { controller = "Home", action = "Topic" },
                new { CategoryUrl = @"[a-zA-Z0-9\-]*", SectionUrl = @"[a-zA-Z0-9\-]*", TopicId = @"[0-9]*", TopicUrl = @"[a-zA-Z0-9\-]*" },
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_SectionPaged",
                AreaUrl + "/{CategoryUrl}/{SectionUrl}/{PageIndex}",
                new { controller = "Home", action = "Section" },
                new { CategoryUrl = @"[a-zA-Z0-9\-]*", SectionUrl = @"[a-zA-Z0-9\-]*", PageIndex = @"[0-9]*" },
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_Section",
                AreaUrl + "/{CategoryUrl}/{SectionUrl}",
                new { controller = "Home", action = "Section" },
                new { CategoryUrl = @"[a-zA-Z0-9\-]*", SectionUrl = @"[a-zA-Z0-9\-]*" },
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_Category",
                AreaUrl + "/{CategoryUrl}",
                new { controller = "Home", action = "Category" },
                new { CategoryUrl = @"[a-zA-Z0-9\-]*" },
                new string[] { AreaNamespace }
            );

            context.MapRoute(
                AreaName + "_Default",
                AreaUrl + "/{controller}/{action}/{id}",
                new { controller = "Home", action = "Index", id = UrlParameter.Optional },
                null, 
                new string[]{AreaNamespace}
            );
        }
    }
}