﻿#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Html;
using Weapsy.Services;
using Weapsy.Apps.Forum.Services;

namespace Weapsy.Apps.Forum.Helpers
{
    public static class _HtmlHelpers
    {
        public static HtmlHelpers WeapsyForum(this HtmlHelper helper)
        {
            return new HtmlHelpers(helper);
        }
    }

    public partial class HtmlHelpers
    {

        #region Private Members

        private static HtmlHelper helper;

        #endregion

        #region Constructors

        public HtmlHelpers(HtmlHelper _helper)
        {
            helper = _helper;
        }

        #endregion

        #region Breadcrumb

        public MvcHtmlString BreadcrumbAdminDashboard()
        {
            var _appService = DependencyResolver.Current.GetService<IAppService>();
            var forumApp = _appService.GetByName("Forum");
            if (forumApp != null)
            {
                return helper.ActionLink(Resources.Apps.Forum.Admin.Breadcrumb_Dashboard, forumApp.AdminAction, forumApp.AdminController, new { area = forumApp.Folder }, null);
            }
            return MvcHtmlString.Create("");
        }

        public MvcHtmlString BreadcrumbAdminCategories()
        {
            var _appService = DependencyResolver.Current.GetService<IAppService>();
            var forumApp = _appService.GetByName("Forum");
            if (forumApp != null)
            {
                return helper.ActionLink(Resources.Apps.Forum.Admin.Breadcrumb_Categories, "Categories", forumApp.AdminController, new { area = forumApp.Folder }, null);
            }
            return MvcHtmlString.Create("");
        }

        public MvcHtmlString BreadcrumbAdminSections()
        {
            var _appService = DependencyResolver.Current.GetService<IAppService>();
            var forumApp = _appService.GetByName("Forum");
            if (forumApp != null)
            {
                return helper.ActionLink(Resources.Apps.Forum.Admin.Breadcrumb_Sections, "Sections", forumApp.AdminController, new { area = forumApp.Folder }, null);
            }
            return MvcHtmlString.Create("");
        }

        public MvcHtmlString BreadcrumbAdminPermissionSets()
        {
            var _appService = DependencyResolver.Current.GetService<IAppService>();
            var forumApp = _appService.GetByName("Forum");
            if (forumApp != null)
            {
                return helper.ActionLink(Resources.Apps.Forum.Admin.Breadcrumb_PermissionSets, "PermissionSets", forumApp.AdminController, new { area = forumApp.Folder }, null);
            }
            return MvcHtmlString.Create("");
        }

        public MvcHtmlString BreadcrumbAdminSettingSets()
        {
            var _appService = DependencyResolver.Current.GetService<IAppService>();
            var forumApp = _appService.GetByName("Forum");
            if (forumApp != null)
            {
                return helper.ActionLink(Resources.Apps.Forum.Admin.Breadcrumb_SettingSets, "SettingSets", forumApp.AdminController, new { area = forumApp.Folder }, null);
            }
            return MvcHtmlString.Create("");
        }

        #endregion

        #region Admin DropDownLists

        public MvcHtmlString CategoryDropDownList(string name, int configurationId, int categoryId)
        {
            var _categoryService = DependencyResolver.Current.GetService<ICategoryService>();
            return helper.DropDownList(name, new SelectList(_categoryService.GetList(configurationId), "Id", "Name"), categoryId);
        }

        public MvcHtmlString PermissionSetDropDownList(string name, int configurationId, int permissionSetId)
        {
            var _permissionSetService = DependencyResolver.Current.GetService<IPermissionSetService>();
            return helper.DropDownList(name, new SelectList(_permissionSetService.GetList(configurationId), "Id", "Name"), permissionSetId);
        }

        public MvcHtmlString SettingSetDropDownList(string name, int configurationId, int settingSetId)
        {
            var _settingSetService = DependencyResolver.Current.GetService<ISettingSetService>();
            return helper.DropDownList(name, new SelectList(_settingSetService.GetList(configurationId), "Id", "Name"), settingSetId);
        }

        #endregion

    }
}