#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Apps.Forum.Entities;

namespace Weapsy.Apps.Forum.Services
{
    public class ConfigurationService : IConfigurationService
    {

        #region Private Members

        private readonly IRepository<ForumConfiguration> _configurationRepository;
        private readonly ICacheManager _cacheManager;

        private const string Key = Consts.GlobalKey + "|Forum|Configuration|SiteId:{0}";

        #endregion

        #region Constructors

        public ConfigurationService(IRepository<ForumConfiguration> configurationRepository,
            ICacheManager cacheManager)
        {
            _configurationRepository = configurationRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(ForumConfiguration entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Configuration");
            }
            _configurationRepository.Create(entity);
            _cacheManager.Remove(Key);
        }

        public virtual void Update(ForumConfiguration entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Configuration");
            }
            _configurationRepository.Update(entity);
            _cacheManager.Remove(Key);
        }

        public virtual void Delete(ForumConfiguration entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("Configuration");
            }
            _configurationRepository.Delete(entity);
            _cacheManager.Remove(Key);
        }

        //private ForumConfiguration GetByName(string name)
        //{
        //    var query = from item in _categoryRepository.Table
        //                where item.Name == name
        //                select item;
        //    return query.FirstOrDefault();
        //}

        public virtual ForumConfiguration GetById(int id)
        {
            var query = from item in _configurationRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        public virtual ForumConfiguration GetBySiteId(int siteId)
        {
            return GetBySiteId(siteId, true);
        }

        public virtual ForumConfiguration GetBySiteId(int siteId, bool useCache)
        {
            ForumConfiguration entity;
            if (useCache)
            {
                string cacheKey = string.Format(Key, siteId.ToString());
                object obj = _cacheManager.Get(cacheKey);
                if (obj != null)
                {
                    entity = (ForumConfiguration)obj;
                }
                else
                {
                    entity = _GetBySiteId(siteId);
                    _cacheManager.Insert(cacheKey, entity, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                entity = _GetBySiteId(siteId);
            }
            return entity;
        }

        public virtual List<ForumConfiguration> GetList(int siteId)
        {
            var query = from item in _configurationRepository.Table
                        where item.SiteId == siteId
                        select item;
            return query.ToList();
        }

        #endregion

        #region Private Methods

        private ForumConfiguration _GetBySiteId(int siteId)
        {
            var query = from item in _configurationRepository.Table
                        where item.SiteId == siteId
                        select item;
            return query.FirstOrDefault();
        }

        #endregion

    }
}