#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Apps.Forum.Entities;

namespace Weapsy.Apps.Forum.Services
{
    public class PermissionSetService : IPermissionSetService
    {

        #region Private Members

        private readonly IRepository<ForumPermissionSet> _permissionSetRepository;
        private readonly ICacheManager _cacheManager;

        private const string Key = Consts.GlobalKey + "|Forum|PermissionSet|Id:{0}";

        #endregion

        #region Constructors

        public PermissionSetService(IRepository<ForumPermissionSet> permissionSetRepository,
            ICacheManager cacheManager)
        {
            _permissionSetRepository = permissionSetRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(ForumPermissionSet entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PermissionSet");
            }
            _permissionSetRepository.Create(entity);
            _cacheManager.Remove(string.Format(Key, entity.Id.ToString()));
        }

        public virtual void Update(ForumPermissionSet entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PermissionSet");
            }
            _permissionSetRepository.Update(entity);
            _cacheManager.Remove(string.Format(Key, entity.Id.ToString()));
        }

        public virtual void Delete(ForumPermissionSet entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("PermissionSet");
            }

            _permissionSetRepository.Delete(entity);
            _cacheManager.Remove(string.Format(Key, entity.Id.ToString()));
        }

        public virtual ForumPermissionSet GetByName(int configurationId, string name)
        {
            var query = from item in _permissionSetRepository.Table
                        where item.ConfigurationId == configurationId && item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        public virtual ForumPermissionSet GetById(int id)
        {
            return GetById(id, true);
        }

        public virtual ForumPermissionSet GetById(int id, bool useCache)
        {
            ForumPermissionSet entity;
            if (useCache)
            {
                string cacheKey = string.Format(Key, id.ToString());
                object obj = _cacheManager.Get(cacheKey);
                if (obj != null)
                {
                    entity = (ForumPermissionSet)obj;
                }
                else
                {
                    entity = _GetById(id);
                    _cacheManager.Insert(cacheKey, entity, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                entity = _GetById(id);
            }
            return entity;
        }

        public virtual List<ForumPermissionSet> GetList(int configurationId)
        {
            var query = from item in _permissionSetRepository.Table
                        where item.ConfigurationId == configurationId
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

        #region Private Methods

        private ForumPermissionSet _GetById(int id)
        {
            var query = from item in _permissionSetRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        #endregion

    }
}