#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Caching;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Data;
using Weapsy.Apps.Forum.Entities;

namespace Weapsy.Apps.Forum.Services
{
    public class SettingSetService : ISettingSetService
    {

        #region Private Members

        private readonly IRepository<ForumSettingSet> _settingSetRepository;
        private readonly ICacheManager _cacheManager;

        private const string Key = Consts.GlobalKey + "|Forum|SettingSet|Id:{0}";

        #endregion

        #region Constructors

        public SettingSetService(IRepository<ForumSettingSet> settingSetRepository,
            ICacheManager cacheManager)
        {
            _settingSetRepository = settingSetRepository;
            _cacheManager = cacheManager;
        }

        #endregion

        #region Public Methods

        public virtual void Create(ForumSettingSet entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("SettingSet");
            }
            _settingSetRepository.Create(entity);
            _cacheManager.Remove(string.Format(Key, entity.Id.ToString()));
        }

        public virtual void Update(ForumSettingSet entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("SettingSet");
            }
            _settingSetRepository.Update(entity);
            _cacheManager.Remove(string.Format(Key, entity.Id.ToString()));
        }

        public virtual void Delete(ForumSettingSet entity)
        {
            if (entity == null)
            {
                throw new ArgumentNullException("SettingSet");
            }

            _settingSetRepository.Delete(entity);
            _cacheManager.Remove(string.Format(Key, entity.Id.ToString()));
        }

        public virtual ForumSettingSet GetByName(int configurationId, string name)
        {
            var query = from item in _settingSetRepository.Table
                        where item.ConfigurationId == configurationId && item.Name == name
                        select item;
            return query.FirstOrDefault();
        }

        public virtual ForumSettingSet GetById(int id)
        {
            return GetById(id, true);
        }

        public virtual ForumSettingSet GetById(int id, bool useCache)
        {
            ForumSettingSet entity;
            if (useCache)
            {
                string cacheKey = string.Format(Key, id.ToString());
                object obj = _cacheManager.Get(cacheKey);
                if (obj != null)
                {
                    entity = (ForumSettingSet)obj;
                }
                else
                {
                    entity = _GetById(id);
                    _cacheManager.Insert(cacheKey, entity, Consts.DefaultCacheTimeOut, Consts.DefaultCacheItemPriority);
                }
            }
            else
            {
                entity = _GetById(id);
            }
            return entity;
        }

        public virtual List<ForumSettingSet> GetList(int configurationId)
        {
            var query = from item in _settingSetRepository.Table
                        where item.ConfigurationId == configurationId
                        orderby item.Name
                        select item;
            return query.ToList();
        }

        #endregion

        #region Private Methods

        private ForumSettingSet _GetById(int id)
        {
            var query = from item in _settingSetRepository.Table
                        where item.Id == id
                        select item;
            return query.FirstOrDefault();
        }

        #endregion

    }
}