#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Modules.Extensions;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using Weapsy.Web.Models;
using Weapsy.Web.Models.Home;
using Weapsy.Web.Modules.Models.Text;

namespace Weapsy.Web.Modules.Controllers
{

	public class TextController : BaseAdminController
    {

        #region Private Members

        private readonly ILanguageService _languageService;
        private readonly IModuleService _moduleService;
        private readonly ITextService _textService;
        
        #endregion

        #region Constructors

        public TextController(ILanguageService languageService, IModuleService moduleService, ITextService textService)
        {
            _languageService = languageService;
            _moduleService = moduleService;
            _textService = textService;
        }

        #endregion

        #region Public Methods
        
        public MvcHtmlString Module(ModuleModel model)
        {
            string text = _textService.GetText(SiteId, model.SharedSettings.Id, WeapsyGlobal.Language.Id);

            if (string.IsNullOrEmpty(text))
            {
                this.Create(model.SharedSettings.Id);
                text = _textService.GetText(SiteId, model.SharedSettings.Id, WeapsyGlobal.Language.Id);
            }

            return MvcHtmlString.Create(text);
        }

        //public JsonResult TextVersion(int id)
        //{
        //    model.Body = "text from json!";
        //    return Json(model, JsonRequestBehavior.AllowGet);
        //} 

        public ActionResult Version(TextSettingsModel model)
        {
            return RedirectToAction("Edit", new { id = model.Text.ModuleId, VersionId = model.SelectedVersionId});
        } 

        public ActionResult Edit(int Id, int VersionId = 0)
        {
            Text entity = _textService.GetByModuleId(SiteId, Id);

            if (entity == null)
            {
                Module module = _moduleService.GetById(SiteId, Id);

                if (module != null)
                {
                    this.Create(Id);

                    entity = new Text();

                    entity = _textService.GetByModuleId(SiteId, Id);
                }
            }

            if (entity == null)
            {
                throw new HttpException(404, "Text Module Not Found");
            }

            List<TextVersion> versions = _textService.GetVersionList(entity.Id);

            TextVersion version = new TextVersion();

            if (VersionId == 0)
            {
                version = versions.Find(x => x.Id == entity.TextVersionId);
            }
            else
            {
                version = versions.Find(x => x.Id == VersionId);
            }

            if (version == null)
            {
                version = new TextVersion();
                version = versions[0];
            }

            //version.Body = HttpUtility.HtmlDecode(version.Body);

            List<LocalizedTextVersionModel> localizedVersions = new List<LocalizedTextVersionModel>();

            foreach (var item in _languageService.GetList(SiteId))
            {
                LocalizedTextVersionModel ltvm = _textService.GetLocalizedVersionByVersionId(version.Id, item.Id).ToModel();
                if (ltvm == null)
                {
                    ltvm = new LocalizedTextVersionModel();
                    ltvm.TextVersionId = version.Id;
                    ltvm.LanguageId = item.Id;
                    LocalizedTextVersion ltve = ltvm.ToEntity();
                    _textService.CreateLocalizedVersion(ltve);
                    ltvm = _textService.GetLocalizedVersionByVersionId(version.Id, item.Id).ToModel();
                    ltvm.LanguageName = item.Name;
                }
                //ltv.Body = HttpUtility.HtmlDecode(ltv.Body);
                localizedVersions.Add(ltvm);
            }

            TextSettingsModel model = new TextSettingsModel();

            model.ModuleTitle = _moduleService.GetById(SiteId, entity.ModuleId).Title;
            model.Text = entity.ToModel();
            model.TextVersion = version.ToModel();
            model.TextVersions = versions.ToModels();
            model.LocalizedTextVersions = localizedVersions;
            model.InsertNewOrUpdate = 1;
            model.SelectedVersionId = version.Id;

            return View(model);
        }

        [ValidateInput(false)]
        [AcceptVerbs(HttpVerbs.Post)]
        public ActionResult Update(TextSettingsModel model)
        {
            Text entity = _textService.GetById(model.Text.Id);

            if (entity == null)
            {
                throw new HttpException(404, "Text Module Not Found");
            }

            if (ModelState.IsValid)
            {
                entity = model.Text.ToEntity(entity);
                entity.SetAuditForUpdate();

                TextVersion versionEntity = new TextVersion();
                List<LocalizedTextVersion> localizedVersionEntities = new List<LocalizedTextVersion>();

                if (model.InsertNewOrUpdate == 1)
                {
                    versionEntity = model.TextVersion.ToEntity();
                    versionEntity.SetAuditForCreate();

                    foreach (var item in model.LocalizedTextVersions)
                    {
                        LocalizedTextVersion ltv = new LocalizedTextVersion();
                        ltv = item.ToEntity(ltv);
                        localizedVersionEntities.Add(ltv);
                    }

                    versionEntity = _textService.CreateVersion(versionEntity, localizedVersionEntities, model.Text.ModuleId);
                }
                else
                {
                    versionEntity = _textService.GetVersionById(model.TextVersion.Id);
                    versionEntity = model.TextVersion.ToEntity(versionEntity);
                    versionEntity.SetAuditForUpdate();

                    foreach (var item in model.LocalizedTextVersions)
                    {
                        LocalizedTextVersion ltv = _textService.GetLocalizedVersionById(item.Id);
                        ltv = item.ToEntity(ltv);
                        localizedVersionEntities.Add(ltv);
                    }

                    _textService.UpdateVersion(versionEntity, localizedVersionEntities);
                }

                Text textToUpdate = _textService.GetById(entity.Id);

                textToUpdate.TextVersionId = versionEntity.Id;

                _textService.Update(textToUpdate);

                return RedirectToAction("List", "Module", new { area = "Admin" });
            }

            return View(model);
        }

        #endregion

        #region Private Methods

        private void Create(int moduleId)
        {
            Text entity = new Text();

            entity = new Text();

            entity.ModuleId = moduleId;
            entity.SetAuditForCreate();

            TextVersion versionEntity = new TextVersion();

            versionEntity.Body = "Your text here.";
            versionEntity.SetAuditForCreate();

            List<LocalizedTextVersion> localizedVersionEntities = new List<LocalizedTextVersion>();

            foreach (var item in _languageService.GetList(SiteId))
            {
                LocalizedTextVersion ltv = new LocalizedTextVersion();
                ltv.LanguageId = item.Id;
                localizedVersionEntities.Add(ltv);
            }

            _textService.Create(entity, versionEntity, localizedVersionEntities);
        }

        #endregion

    }
}