#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using Weapsy.Core.Net;
using Weapsy.Web.Framework.DataAnnotations;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Extensions;
using Weapsy.Web.Framework;
using Weapsy.Web.Framework.Controllers;
using Weapsy.Web.Models;
using Weapsy.Web.Models.Account;

namespace Weapsy.Web.Controllers
{
    public class AccountController : BaseAccountController
    {

        #region Private Members

        private readonly IAppService _appService;
        private readonly IPluginService _pluginService;

        #endregion

        #region Constructors

        public AccountController(IAppService appService,
            IPluginService pluginService)
        {
            _appService = appService;
            _pluginService = pluginService;
        }

        #endregion

        #region Public Methods

        public ActionResult Index()
        {
            IndexModel model = new IndexModel();

            var pluginEntities =_pluginService.GetSitePlugins(SiteId, 
                PluginType.Account, 
                WeapsyGlobal.Language.Id);

            var pluginModels = new List<PluginModel>();

            foreach (var plugin in pluginEntities.ToModels())
            {
                if (plugin.Active && Utils.HasPermission(plugin.Authorized))
                {
                    pluginModels.Add(plugin);
                }
            }

            foreach (var plugin in pluginModels)
            {
                if (plugin.PluginDefinitionAppId > 0)
                {
                    var app =_appService.GetById(plugin.PluginDefinitionAppId);
                    if (app != null)
                    {
                        plugin.Area = app.Folder;
                        plugin.ImagePrefix = "~/areas/" + app.Folder + "/";
                    }
                }
                else
                {
                    plugin.Area = "Plugins";
                    plugin.ImagePrefix = "~/content/images/";
                }
            }

            model.Plugins = pluginModels;

            return View(model);
        }

        #endregion

    }
}