#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Web.Mvc;
using System.Configuration;
using Weapsy.Entities;
using Weapsy.Services;
using Recaptcha;

namespace Weapsy.Web.Framework.DataAnnotations
{

	public class ReCaptchaValidatorAttribute : ActionFilterAttribute
	{
		private const string ChallengeFieldKey = "recaptcha_challenge_field";
		private const string ResponseFieldKey = "recaptcha_response_field";

        public enum CaptchaType
        { 
            Register = 1,
            Login = 2,
            PasswordRecovery = 3
        }

        private CaptchaType _type { get; set; }

        public ReCaptchaValidatorAttribute(CaptchaType type)
        {
            _type = type;
        }

		public override void OnActionExecuting(ActionExecutingContext filterContext)
		{
            bool valid = false;

            string challengeFieldValue = filterContext.HttpContext.Request.Form[ChallengeFieldKey];
            string responseFieldValue = filterContext.HttpContext.Request.Form[ResponseFieldKey];

            if (!string.IsNullOrEmpty(challengeFieldValue) && !string.IsNullOrEmpty(responseFieldValue))
            {
                bool enabled = false;

                var _settingService = DependencyResolver.Current.GetService<ISettingService>();
                SecuritySettings _securitySettings = _settingService.GetSettings<SecuritySettings>();

                switch (_type)
                {
                    case CaptchaType.Login:
                        enabled = _securitySettings.LoginCaptcha;
                        break;
                    case CaptchaType.PasswordRecovery:
                        enabled = _securitySettings.PasswordRecoveryCaptcha;
                        break;
                    case CaptchaType.Register:
                        enabled = _securitySettings.RegisterCaptcha;
                        break;
                }

                if (enabled)
                {
                    RecaptchaValidator validator = new RecaptchaValidator
                    {
                        PrivateKey = _securitySettings.ReCaptchaPrivateKey,
                        RemoteIP = filterContext.HttpContext.Request.UserHostAddress,
                        Challenge = challengeFieldValue,
                        Response = responseFieldValue
                    };

                    RecaptchaResponse response = validator.Validate();
                    valid = response.IsValid;
                }
            }

            filterContext.ActionParameters["captchaValid"] = valid;
            base.OnActionExecuting(filterContext); 
		}

	}
}