#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Caching;
using System.Web.Mvc;
using Weapsy.Entities;

namespace Weapsy.Web.Framework
{

	public static class Extensions
	{

        public static SelectList ToSelectList<TEnum>(this TEnum enumObj) where TEnum : struct
        {
            var values = from TEnum e in Enum.GetValues(typeof(TEnum))
                         select new { Id = Convert.ToInt32(e), Name = e.ToString() };

            return new SelectList(values, "Id", "Name", enumObj);
        }

        public static Dictionary<string, BaseEntity> ToDictionary(this List<BaseEntity> entities)
        {
            Dictionary<string, BaseEntity> dic = new Dictionary<string, BaseEntity>();
            foreach (BaseEntity item in entities)
            {
                dic.Add(item.Id.ToString(), item);
            }
            return dic;
        }

        public static void SetAuditForCreate(this BaseAuditEntity entity)
        {
            entity.CreatedByUserId = WeapsyGlobal.Current.User.Id;
            entity.CreatedByIpAddress = HttpContext.Current.Request.UserHostAddress;
            entity.CreatedOnDate = Utils.GetDateForDB();

            entity.ModifiedByUserId = WeapsyGlobal.Current.User.Id;
            entity.ModifiedByIpAddress = HttpContext.Current.Request.UserHostAddress;
            entity.ModifiedOnDate = Utils.GetDateForDB();
        }

        public static void SetAuditForUpdate(this BaseAuditEntity entity)
        {
            entity.ModifiedByUserId = WeapsyGlobal.Current.User.Id;
            entity.ModifiedByIpAddress = HttpContext.Current.Request.UserHostAddress;
            entity.ModifiedOnDate = Utils.GetDateForDB();
        }

	}

}