﻿#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using System.Collections.Generic;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Text;
using System.Web.UI;
using System.Web.Mvc;
using System.Web.Mvc.Html;
using System.Web.Routing;
using Weapsy;
using Weapsy.Core.Caching;
using Weapsy.Entities;
using Weapsy.Services;
using Weapsy.Web.Admin.Models;
using Telerik.Web.Mvc.UI;

namespace Weapsy.Web.Framework.Helpers
{

    public static class _HtmlHelpers
    {
        public static HtmlHelpers Weapsy(this HtmlHelper helper)
        {
            return new HtmlHelpers(helper);
        }
    }

    public partial class HtmlHelpers
    {

        private static HtmlHelper helper;

        public HtmlHelpers(HtmlHelper _helper)
        {
            helper = _helper;
        }

        public MvcHtmlString AppUrl(string name)
        {
            string result = name;

            var _appService = DependencyResolver.Current.GetService<IAppService>();

            App app = _appService.GetByName(name);

            if (app != null)
            {
                result = app.Url;
            }

            return MvcHtmlString.Create('"' + result + '"');
        }

        public MvcHtmlString AppFolder(string name)
        {
            string result = name;

            var _appService = DependencyResolver.Current.GetService<IAppService>();

            App app = _appService.GetByName(name);

            if (app != null)
            {
                result = app.Folder;
            }

            return MvcHtmlString.Create(result);
        }

        public MvcHtmlString EditLink(string action, object routeValues, string alt)
        {
            UrlHelper urlHelper = new UrlHelper(helper.ViewContext.RequestContext);

            // build the <img> tag
            var imgBuilder = new TagBuilder("img");
            imgBuilder.MergeAttribute("src", urlHelper.Content(""));
            imgBuilder.MergeAttribute("alt", alt);
            string imgHtml = imgBuilder.ToString(TagRenderMode.SelfClosing);

            // build the <a> tag
            var anchorBuilder = new TagBuilder("a");
            anchorBuilder.MergeAttribute("href", urlHelper.Action(action, routeValues));
            anchorBuilder.InnerHtml = imgHtml; // include the <img> tag inside
            string anchorHtml = anchorBuilder.ToString(TagRenderMode.Normal);

            return MvcHtmlString.Create(anchorHtml);
        }

        public MvcHtmlString TelerikSiteMenu()
        {
            //IKernel kernel = new StandardKernel(new InjectionModule());

            var _languageService = DependencyResolver.Current.GetService<ILanguageService>();
            var _pageService = DependencyResolver.Current.GetService<IPageService>();

            var context = WeapsyGlobal.Current;

            string pageUrl = "";
            string subPageUrl = "";
            string languagePrefix = context.Language.Url + "/";
            bool useLocalizedUrls = _languageService.GetList(WeapsyGlobal.Current.Site.Id).Count > 1
                && context.Site.LanguageSelectionEnabled
                && context.Site.LanguageUrlEnabled;
            
            var menu = helper.Telerik().Menu().Name("TelerikSiteMenu").Items(x =>
            {
                foreach (var page in WeapsyGlobal.Current.Pages.FindAll(p => p.ParentId == 0 && p.MainMenu == true))
                {
                    pageUrl = "none";

                    if (!page.Disabled)
                    {
                        if (!string.IsNullOrWhiteSpace(page.Link))
                        {
                            pageUrl = page.Link;
                        }
                        else
                        {
                            if (useLocalizedUrls && page.LanguageUrlEnabled)
                            {
                                pageUrl = "~/" + languagePrefix + page.LocalizedUrl;
                            }
                            else
                            {
                                pageUrl = "~/" + page.Url;
                            } 
                        }
                    }

                    x.Add().Text(page.LocalizedMenuText).Url(pageUrl).HtmlAttributes(new { title = page.LocalizedMenuTitle}).Items(y =>
                        {
                            foreach (var subPage in WeapsyGlobal.Current.Pages.FindAll(p => p.ParentId == page.Id && p.MainMenu == true))
                            {
                                subPageUrl = "none";

                                if (!subPage.Disabled)
                                {
                                    if (!string.IsNullOrWhiteSpace(subPage.Link))
                                    {
                                        subPageUrl = subPage.Link;
                                    }
                                    else
                                    {
                                        if (useLocalizedUrls && subPage.LanguageUrlEnabled)
                                        {
                                            subPageUrl = "~/" + languagePrefix + subPage.LocalizedUrl;
                                        }
                                        else
                                        {
                                            subPageUrl = "~/" + subPage.Url;
                                        }
                                    }
                                }

                                y.Add().Text(subPage.LocalizedMenuText).Url(subPageUrl).HtmlAttributes(new { title = subPage.LocalizedMenuTitle });
                            }
                        });
                }
            });

            return MvcHtmlString.Create(menu.ToHtmlString());
        }

        public MvcHtmlString TelerikAdminMenu()
        {
            var _appService = DependencyResolver.Current.GetService<IAppService>();

            UrlHelper urlHelper = new UrlHelper(helper.ViewContext.RequestContext);

            var menu = helper.Telerik().Menu().Name("TelerikAdminMenu").Items(x =>
            {
                x.Add().Text("Dashboard").Url(urlHelper.Action("Index", "Home")).HtmlAttributes(new { title = "Dashboard" });
                x.Add().Text("Pages").Url(urlHelper.Action("List", "Page")).HtmlAttributes(new { title = "Pages" });
                x.Add().Text("Modules").Url(urlHelper.Action("List", "Module")).HtmlAttributes(new { title = "Modules" });
                x.Add().Text("Apps").HtmlAttributes(new { title = "Apps" }).Items(y =>
                    {
                        foreach (var item in _appService.GetList())
                        {
                            //y.Add().Text(item.Name).Url(urlHelper.Action(item.AdminAction, item.AdminController, new { area = item.Name }, null)).HtmlAttributes(new { title = item.Name });
                        }
                    });
                x.Add().Text("Site").HtmlAttributes(new { title = "Site" }).Items(y =>
                {
                    y.Add().Text("Settings").Url(urlHelper.Action("Settings", "Site")).HtmlAttributes(new { title = "Settings" });
                    y.Add().Text("Plugins").Url(urlHelper.Action("List", "Plugin")).HtmlAttributes(new { title = "Plugins" });
                    y.Add().Text("Widgets").Url(urlHelper.Action("List", "Widget")).HtmlAttributes(new { title = "Widgets" });
                    y.Add().Text("Languages").Url(urlHelper.Action("List", "Language")).HtmlAttributes(new { title = "Languages" });
                });
                x.Add().Text("Members").HtmlAttributes(new { title = "Members" }).Items(y =>
                {
                    y.Add().Text("Users").Url(urlHelper.Action("List", "User")).HtmlAttributes(new { title = "Users" });
                    y.Add().Text("Groups").Url(urlHelper.Action("List", "Group")).HtmlAttributes(new { title = "Groups" });
                    y.Add().Text("Roles").Url(urlHelper.Action("List", "Role")).HtmlAttributes(new { title = "Roles" });
                });
                x.Add().Text("Appearance").HtmlAttributes(new { title = "Appearance" }).Items(y =>
                {
                    y.Add().Text("Themes").Url(urlHelper.Action("List", "Theme")).HtmlAttributes(new { title = "Themes" });
                    y.Add().Text("PageTemplates").Url(urlHelper.Action("List", "PageTemplate")).HtmlAttributes(new { title = "Page Templates" });
                    y.Add().Text("ModuleTemplates").Url(urlHelper.Action("List", "ModuleTemplate")).HtmlAttributes(new { title = "Module Templates" });
                });
                x.Add().Text("Advanced").HtmlAttributes(new { title = "Advanced" }).Items(y =>
                {
                    y.Add().Text("Settings").Url(urlHelper.Action("Settings", "Advanced")).HtmlAttributes(new { title = "Settings" });
                    y.Add().Text("Apps").Url(urlHelper.Action("List", "App")).HtmlAttributes(new { title = "Apps" });
                    y.Add().Text("Module Definitions").Url(urlHelper.Action("List", "ModuleDefinition")).HtmlAttributes(new { title = "Module Definitions" });
                    y.Add().Text("Plugin Definitions").Url(urlHelper.Action("List", "PluginDefinition")).HtmlAttributes(new { title = "Plugin Definitions" });
                    y.Add().Text("Widget Definitions").Url(urlHelper.Action("List", "WidgetDefinition")).HtmlAttributes(new { title = "Widget Definitions" });
                });
            });

            return MvcHtmlString.Create(menu.ToHtmlString());
        }

        public MvcHtmlString ProfileLink(User user)
        {
            return helper.RouteLink(user.DisplayName, "Profile_Default", new { ProfileName = user.ProfileName });
        }

        public MvcHtmlString SecurityCheckBoxList(string name, string roles, bool IncludeAll, bool IncludeRegistered, bool IncludeAnonymous)
        {
            Dictionary<string, string> dic = new Dictionary<string, string>();
            if (IncludeAll) dic.Add(Consts.RoleAll, Resources.GlobalStrings.RoleAll);
            if (IncludeRegistered) dic.Add(Consts.RoleRegistered, Resources.GlobalStrings.RoleRegistered);
            if (IncludeAnonymous) dic.Add(Consts.RoleAnonymous, Resources.GlobalStrings.RoleAnonymous);
            foreach (string x in System.Web.Security.Roles.GetAllRoles())
            {
                dic.Add(x, x);
            }

            StringBuilder builder = new StringBuilder();

            foreach (KeyValuePair<string, string> kvp in dic)
            {
                builder.Append(helper.CheckBox(name + kvp.Key, Utils.CheckBoxChecked(kvp.Key, roles), Utils.RoleCheckBoxDisabled(kvp.Key)));
                builder.Append(helper.Label(name + kvp.Key, kvp.Value));
                builder.Append("<br />");
            }

            return MvcHtmlString.Create(builder.ToString());
        }

        public MvcHtmlString RoleCheckBoxList(string membershipName)
        {
            StringBuilder builder = new StringBuilder();

            foreach (string role in System.Web.Security.Roles.GetAllRoles())
            {
                bool flag = false;

                if (!string.IsNullOrWhiteSpace(membershipName))
                {
                    flag = Utils.CheckBoxChecked(role, System.Web.Security.Roles.GetRolesForUser(membershipName));
                }

                builder.Append(helper.CheckBox("Role-" + role, flag));
                builder.Append(helper.Label("Role-" + role, role));
                builder.Append("<br />");
            }

            return MvcHtmlString.Create(builder.ToString());
        }

        public MvcHtmlString AppDropDownList(string name, int appId)
        {
            var _appService = DependencyResolver.Current.GetService<IAppService>();
            return helper.DropDownList(name, new SelectList(_appService.GetDictionary(), "key", "value"), appId);
        }

        public MvcHtmlString MailTemplateTypeDropDownList(string name, int mailTemplateTypeId)
        {
            var _mailTemplateTypeService = DependencyResolver.Current.GetService<IMailTemplateTypeService>();
            return helper.DropDownList(name, new SelectList(_mailTemplateTypeService.GetList(), "Id", "Name"), mailTemplateTypeId);
        }

        public MvcHtmlString ModuleDefinitionDropDownList(string name, int moduleDefinitionId)
        {
            var _moduleDefinitionService = DependencyResolver.Current.GetService<IModuleDefinitionService>();
            return helper.DropDownList(name, new SelectList(_moduleDefinitionService.GetDictionary(), "key", "value"), moduleDefinitionId);  
        }

        public MvcHtmlString LanguageDropDownList(string name, int languageId)
        {
            var _languageService = DependencyResolver.Current.GetService<ILanguageService>();
            return helper.DropDownList(name, new SelectList(_languageService.GetDictionary(WeapsyGlobal.Current.Site.Id), "key", "value"), languageId);
        }

        public MvcHtmlString PageDropDownList(string name, int pageId)
        {
            var _pageService = DependencyResolver.Current.GetService<IPageService>();
            return helper.DropDownList(name, new SelectList(_pageService.GetDictionary(WeapsyGlobal.Current.Site.Id, WeapsyGlobal.Current.Language.Id), "key", "value"), pageId);
        }

        public MvcHtmlString ParentPageDropDownList(string name, int pageId, int parentId)
        {
            var _pageService = DependencyResolver.Current.GetService<IPageService>();
            return helper.DropDownList(name, new SelectList(_pageService.GetDictionaryForParent(WeapsyGlobal.Current.Site.Id, WeapsyGlobal.Current.Language.Id, pageId), "key", "value"), parentId);
        }

        public MvcHtmlString PageTemplateDropDownList(string name, int pageTemplateId, bool includeInherit)
        {
            var _pageTemplateService = DependencyResolver.Current.GetService<IPageTemplateService>();
            return helper.DropDownList(name, new SelectList(_pageTemplateService.GetDictionary(includeInherit), "key", "value"), pageTemplateId);
        }

        //public MvcHtmlString PluginTypeDropDownList(string name, int typeId)
        //{
        //    var _pluginDefinitionService = DependencyResolver.Current.GetService<IPluginDefinitionService>();
        //    return helper.DropDownList(name, new SelectList(_pluginDefinitionService.GetTypes(), "key", "value"), typeId);
        //}

        public MvcHtmlString PluginDefinitionDropDownList(string name, int definitionId)
        {
            var _pluginDefinitionService = DependencyResolver.Current.GetService<IPluginDefinitionService>();
            return helper.DropDownList(name, new SelectList(_pluginDefinitionService.GetList(), "Id", "Name"), definitionId);
        }

        //public MvcHtmlString WidgetTypeDropDownList(string name, int widgetId)
        //{
        //    var _widgetDefinitionService = DependencyResolver.Current.GetService<IWidgetDefinitionService>();
        //    return helper.DropDownList(name, new SelectList(_widgetDefinitionService.GetTypes(), "key", "value"), widgetId);
        //}

        public MvcHtmlString WidgetDefinitionDropDownList(string name, int definitionId)
        {
            var _widgetDefinitionService = DependencyResolver.Current.GetService<IWidgetDefinitionService>();
            return helper.DropDownList(name, new SelectList(_widgetDefinitionService.GetList(), "Id", "Name"), definitionId);
        }

        public MvcHtmlString EmailAccountDropDownList(string name, int emailAccountId)
        {
            var _emailAccountService = DependencyResolver.Current.GetService<IEmailAccountService>();
            return helper.DropDownList(name, new SelectList(_emailAccountService.GetList(WeapsyGlobal.Current.Site.Id), "Id", "Address"), emailAccountId);
        }

        public MvcHtmlString MailTemplateDropDownList(string name, int mailTemplateId)
        {
            var _mailTemplateService = DependencyResolver.Current.GetService<IMailTemplateService>();
            return helper.DropDownList(name, new SelectList(_mailTemplateService.GetList(WeapsyGlobal.Current.Site.Id), "Id", "Name"), mailTemplateId);
        }

        public MvcHtmlString ModuleTemplateDropDownList(string name, int moduleTemplateId, bool includeInherit)
        {
            var _moduleTemplateService = DependencyResolver.Current.GetService<IModuleTemplateService>();
            return helper.DropDownList(name, new SelectList(_moduleTemplateService.GetDictionary(includeInherit), "key", "value"), moduleTemplateId);
        }

        public MvcHtmlString ThemeDropDownList(string name, int themeId, bool includeInherit)
        {
            var _themeService = DependencyResolver.Current.GetService<IThemeService>();
            return helper.DropDownList(name, new SelectList(_themeService.GetDictionary(includeInherit), "key", "value"), themeId);
        }

        public MvcHtmlString AddExsistingModuleDropDownList(int siteId, int excludePageId)
        {
            var _moduleService = DependencyResolver.Current.GetService<IModuleService>();
            return helper.DropDownList("PageSettings.ModuleId", new SelectList(_moduleService.GetListForExisting(siteId, excludePageId), "Id", "Title"));
        }

        public MvcHtmlString Zone(Web.Models.Home.ZoneModel model)
        {
            StringBuilder builder = new StringBuilder();
            foreach (Web.Models.Home.ModuleModel item in model.Modules)
            {
                builder.Append(helper.Partial("../Shared/ModuleTemplates/" + item.ModuleTemplate.ViewName, item));
            }
            return MvcHtmlString.Create(builder.ToString());
        }

        public MvcHtmlString Zones(List<Web.Models.Home.ZoneModel> model)
        {
            StringBuilder builder = new StringBuilder();
            foreach (var zone in model)
            {
                foreach (var item in zone.Modules)
                {
                    builder.Append(helper.Partial("../Shared/ModuleTemplates/" + item.ModuleTemplate.ViewName, item));
                }
            }
            return MvcHtmlString.Create(builder.ToString());
        }

        public MvcHtmlString Module(Web.Models.Home.ModuleModel model)
        {
            if (model.Definition.AppId > 0)
            {
                App app = DependencyResolver.Current.GetService<IAppService>().GetById(model.Definition.AppId);

                RouteValueDictionary rvd = new RouteValueDictionary();

                rvd.Add("area", app.Url);
                rvd.Add("settings", model);

                return helper.Action(
                    model.Definition.Action,
                    model.Definition.Controller,
                    rvd);
            }
            else
            {
                return helper.Action(
                    model.Definition.Action,
                    model.Definition.Controller,
                    model);
            }
        }

        public MvcHtmlString ManageModuleLink(ModuleModel model)
        {
            string areaUrl = "Modules";

            if (model.ModuleDefinitionAppId > 0)
            {
                var _appService = DependencyResolver.Current.GetService<IAppService>();
                App app = _appService.GetById(model.ModuleDefinitionAppId);
                areaUrl = app.Url;
            }

            return helper.ActionLink(" ",
                model.ModuleDefinitionAdminAction,
                model.ModuleDefinitionAdminController,
                new { Id = model.Id, area = areaUrl },
                new { @class = "manage" });
        }

        public MvcHtmlString ManagePluginLink(PluginModel model)
        {
            string areaUrl = "";

            if (model.PluginDefinitionAppId > 0)
            {
                var _appService = DependencyResolver.Current.GetService<IAppService>();
                App app = _appService.GetById(model.PluginDefinitionAppId);
                areaUrl = app.Url;
            }

            return helper.ActionLink("Manage",
                model.PluginDefinitionAdminAction,
                model.PluginDefinitionAdminController,
                new { Id = model.Id, area = areaUrl },
                null);
        }

        public MvcHtmlString ManageWidgetLink(WidgetModel model)
        {
            string areaUrl = "";

            if (model.WidgetDefinitionAppId > 0)
            {
                var _appService = DependencyResolver.Current.GetService<IAppService>();
                App app = _appService.GetById(model.WidgetDefinitionAppId);
                areaUrl = app.Url;
            }

            return helper.ActionLink("Manage",
                model.WidgetDefinitionAdminAction,
                model.WidgetDefinitionAdminController,
                new { Id = model.Id, area = areaUrl },
                null);
        }

        public MvcHtmlString CachePriorityDropDownList(string name, int priority)
        {
            var _cacheManager = DependencyResolver.Current.GetService<ICacheManager>();
            return helper.DropDownList(name, new SelectList(_cacheManager.CacheItemPriorities(), "key", "value"), priority);
        }

        public MvcHtmlString SiteTitle()
        {
            Site site = WeapsyGlobal.Current.Site;

            string result = site.Name;

            if (!string.IsNullOrWhiteSpace(site.Logo))
            {
                UrlHelper urlHelper = new UrlHelper(helper.ViewContext.RequestContext);
                var imgBuilder = new TagBuilder("img");
                imgBuilder.MergeAttribute("src", urlHelper.Content(Path.Combine(AppSettings.FolderForFiles, site.Logo)));
                imgBuilder.MergeAttribute("alt", site.Title);
                imgBuilder.MergeAttribute("title", site.Title);
                result = imgBuilder.ToString(TagRenderMode.SelfClosing);
            }
            else
            {
                if (!string.IsNullOrWhiteSpace(site.Title))
                {
                    result = site.Title;
                }
            }

            return MvcHtmlString.Create(result);
        }

        public MvcHtmlString SiteCopyright()
        {
            string result = WeapsyGlobal.Current.Site.Copyright;

            //TODO:LOCALIZE COPYRIGHT TEXT

            return MvcHtmlString.Create(result);
        }

        public MvcHtmlString Breadcrumb()
        {
            // ...
            return MvcHtmlString.Create("");
        }

        public MvcHtmlString BreadcrumbAdminDashboard()
        {
            return helper.ActionLink(Resources.GlobalStrings.BreadcrumbDashboard, "Index", "Home", new { area = "Admin" }, null);
        }

        public MvcHtmlString BreadcrumbSeparator()
        {
            return MvcHtmlString.Create(Resources.GlobalStrings.BreadcrumbSeparator);
        }

        public MvcHtmlString WhiteSpace()
        {
            return WhiteSpace(1);
        }
        public MvcHtmlString WhiteSpace(int count)
        {
            string nbsp = "";
            for (int i = 1; i <= count; i++)
            {
                nbsp += "&nbsp;";
            }
            return MvcHtmlString.Create(nbsp);
        }

        public string ReCaptcha()
        {
            var _settingsService = DependencyResolver.Current.GetService<ISettingService>();
            SecuritySettings securitySettings = _settingsService.GetSettings<SecuritySettings>();

            //http://wiki.recaptcha.net/index.php/Theme
            var captchaControl = new Recaptcha.RecaptchaControl
            {
                ID = "recaptcha",
                Theme = "clean",
                PublicKey = securitySettings.ReCaptchaPublicKey,
                PrivateKey = securitySettings.ReCaptchaPrivateKey
            };
            var htmlWriter = new HtmlTextWriter(new StringWriter());
            captchaControl.RenderControl(htmlWriter);
            return htmlWriter.InnerWriter.ToString();
        }

        public string Truncate(string input, int length)
        {
            if (input.Length <= length)
            {
                return input;
            }
            else
            {
                return input.Substring(0, length) + "...";
            }
        }

        //=============================================================================
        //  Thanks to ??? : http://???
        //=============================================================================

        public string Pager(int currentPage, int pageSize, int totalItemCount, int totalPages, string urlPrefix, bool UseJQuery)
        {
            if (!(totalItemCount > 0))
            {
                return string.Empty;
            }

            int pageCount = Convert.ToInt32(Math.Ceiling(totalItemCount / Convert.ToDouble(pageSize)));
            const int nrOfPagesToDisplay = 10;

            var sb = new StringBuilder();

            sb.Append("<div class=\"pager\">");

            // Previous
            sb.Append(currentPage > 1 ? GeneratePageLink(urlPrefix, currentPage - 1, "&lt;", UseJQuery) : "<span class=\"disabled\">&lt;</span>&nbsp;");

            int start = 1;
            int end = pageCount;

            if (pageCount > nrOfPagesToDisplay)
            {
                int middle = Convert.ToInt32(Math.Ceiling(nrOfPagesToDisplay / 2.0)) - 1;
                int below = (currentPage - middle);
                int above = (currentPage + middle);

                if (below < 4)
                {
                    above = nrOfPagesToDisplay;
                    below = 1;
                }
                else if (above > (pageCount - 4))
                {
                    above = pageCount;
                    below = (pageCount - nrOfPagesToDisplay);
                }

                start = below;
                end = above;
            }

            if (start > 3)
            {
                sb.Append(GeneratePageLink(urlPrefix, 1, "1", UseJQuery));
                sb.Append(GeneratePageLink(urlPrefix, 2, "2", UseJQuery));
                sb.Append("...");
            }

            for (int i = start; i <= end; i++)
            {
                if (i == currentPage || (currentPage <= 0 && i == 0))
                {
                    sb.AppendFormat("<span class=\"current\">{0}</span>&nbsp;", i);
                }
                else
                {
                    sb.Append(GeneratePageLink(urlPrefix, i, i.ToString(), UseJQuery));
                }
            }
            if (end < (pageCount - 3))
            {
                sb.Append("...");
                sb.Append(GeneratePageLink(urlPrefix, pageCount - 1, (pageCount - 1).ToString(), UseJQuery));
                sb.Append(GeneratePageLink(urlPrefix, pageCount, pageCount.ToString(), UseJQuery));
            }

            // Next
            sb.Append(currentPage < pageCount ? GeneratePageLink(urlPrefix, (currentPage + 1), "&gt;", UseJQuery) : "<span class=\"disabled\">&gt;</span>&nbsp;");

            sb.Append("</div>");

            return sb.ToString();
        }

        private static string GeneratePageLink(string urlPrefix, int pageNumber, string text, bool UsejQuery)
        {
            if (UsejQuery)
            {
                return string.Format("<a href=\"{0}{1}\" onclick=\"ChangePage({1})\" title=\"Page {1}\">{2}</a>", urlPrefix, pageNumber, text);
            }
            else
            {
                return string.Format("<a href=\"{0}{1}\" title=\"Page {1}\">{2}</a>", urlPrefix, pageNumber, text);
            }
        }

        public MvcHtmlString AccountMenu()
        {
            //UrlHelper urlHelper = new UrlHelper(helper.ViewContext.RequestContext);

            var _appService = DependencyResolver.Current.GetService<IAppService>();
            var _pluginService = DependencyResolver.Current.GetService<IPluginService>();

            StringBuilder builder = new StringBuilder();

            builder.Append("<ul>");

            foreach (var item in _pluginService.GetSitePlugins(WeapsyGlobal.Current.Site.Id, 
                PluginType.Account, 
                WeapsyGlobal.Current.Language.Id))
            {
                string areaName = "Plugins";

                if (item.PluginDefinition.AppId > 0)
                {
                    areaName = _appService.GetById(item.PluginDefinition.AppId).Folder;
                }

                builder.Append("<li>");

                if (!string.IsNullOrWhiteSpace(item.PluginDefinition.MenuIcon))
                { 
                    //ADD ICON
                }

                builder.Append(helper.ActionLink(item.LocalizedMenuText,
                    item.PluginDefinition.Action,
                    item.PluginDefinition.Controller,
                    new { area = areaName },
                    new { title = item.LocalizedMenuTitle }));
                builder.Append("</li>");
            }

            builder.Append("</ul>");

            return MvcHtmlString.Create(builder.ToString());
        }

    } 
}