#region Copyright

//  Weapsy - http://www.weapsy.org
//  Copyright (c) 2011-2012 Luca Cammarata Briguglia
//  Licensed under the Weapsy Public License Version 1.0 (WPL-1.0)
//  A copy of this license may be found at http://www.weapsy.org/Documentation/License.txt

#endregion

using System;
using Microsoft.Web.Infrastructure.DynamicModuleHelper;
using Weapsy.Core.Caching;
using Weapsy.Core.Net;
using Weapsy.Data;
using Weapsy.Entities;
using Weapsy.Services;
using Autofac;
using Autofac.Integration.Mvc;

namespace Weapsy.Web.Framework
{
    public class InjectionModule : Autofac.Module
    {
        protected override void Load(ContainerBuilder builder)
        {
            if (builder == null) 
                throw new ArgumentNullException("builder");

            builder.RegisterType<CacheManager>().As<ICacheManager>();
            builder.RegisterType<MailManager>().As<IMailManager>();

            builder.RegisterType<WeapsyContext>().As<IDbContext>();

            builder.RegisterGeneric(typeof(EfRepository<>)).As(typeof(IRepository<>));

            builder.RegisterType<AppService>().As<IAppService>();
            builder.RegisterType<EmailAccountService>().As<IEmailAccountService>();
            builder.RegisterType<InstallService>().As<IInstallService>();
            builder.RegisterType<LanguageService>().As<ILanguageService>();
            builder.RegisterType<MailTemplateService>().As<IMailTemplateService>();
            builder.RegisterType<MailTemplateTypeService>().As<IMailTemplateTypeService>();
            builder.RegisterType<ModuleService>().As<IModuleService>();
            builder.RegisterType<ModuleDefinitionService>().As<IModuleDefinitionService>();
            builder.RegisterType<ModuleTemplateService>().As<IModuleTemplateService>();
            builder.RegisterType<PageService>().As<IPageService>();
            builder.RegisterType<PageTemplateService>().As<IPageTemplateService>();
            builder.RegisterType<PluginService>().As<IPluginService>();
            builder.RegisterType<PluginDefinitionService>().As<IPluginDefinitionService>();
            builder.RegisterType<RoleService>().As<IRoleService>();
            builder.RegisterType<SettingService>().As<ISettingService>();
            builder.RegisterType<SiteService>().As<ISiteService>();
            builder.RegisterType<TextService>().As<ITextService>();
            builder.RegisterType<ThemeService>().As<IThemeService>();
            builder.RegisterType<UserService>().As<IUserService>();
            builder.RegisterType<WidgetService>().As<IWidgetService>();
            builder.RegisterType<WidgetDefinitionService>().As<IWidgetDefinitionService>();
        }
    }
}